/*
 * The openStar project: stellar interior structure 
 *
 * PolyStar
 * V 2.0
 * June 2021
 * JQuery version
 * 
 * C. Ian Short
 * Saint Mary's University
 * Department of Astronomy and Physics
 * Institute for Computational Astrophysics (ICA)
 * Halifax, NS, Canada
 *  * ian.short@smu.ca
 * www.ap.smu.ca/~ishort/
 *
 * 
 * Open source pedagogical computational stellar astrophysics
 *
* Polytrope with polytropic constant, K, determined as part of the
* modelling process
* Eddington standard model for star of ideal gas + photon mixture
*
* See Stellar Astrophysics, 2nd Ed., Ch. 19, by Kippenhahn, Weigert & Weiss
* (Springer)
 
 * 
 * System requirements for JavaScript version: JavaScript intrepretation enabld in WWW browser (usually by default)
 *
 * Code provided "as is" - there is no formal support 
 *
 * Java default license applies:
 * End users may adapt, modify, develop, debug, and deploy at will
 * for academic and othe non-profit uses, but are asked to leave this
 * header text in place (although they may add to the header text).
 *
 * The MIT License (MIT)
 * Copyright (c) 2016 C. Ian Short 
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
*
 */


// **********************************************

"use strict"; //testing only!

// Global variables - Doesn't work - scope not global!

var c = 2.9979249E+10; // light speed in vaccuum in cm/s
var sigma = 5.670373E-5; //Stefan-Boltzmann constant ergs/s/cm^2/K^4
var radConst = 4.0E0 * sigma / c;  //radiation constant
var wien = 2.8977721E-1; // Wien's displacement law constant in cm K
var kBoltz = 1.3806488E-16; // Boltzmann constant in ergs/K
var h = 6.62606957E-27; //Planck's constant in ergs sec
var ee = 4.80320425E-10; //fundamental charge unit in statcoulombs (cgs)
var mE = 9.10938291E-28; //electron mass (g)
var GConst = 6.674e-8; //Newton's gravitational constant (cgs)
var RConst = 8.314E7;  //Universal gas constant
//Conversion factors
var amu = 1.66053892E-24; // atomic mass unit in g
var eV = 1.602176565E-12; // eV in ergs
var rSun = 6.955e10; // solar radii to cm
var mSun = 1.9891e33; // solar masses to g
var lSun = 3.846e33; // solar bolometric luminosities to ergs/s
var au = 1.4960e13; // 1 AU in cm

var rEarth = 6.371e8;  //# cm
var logREarth = Math.log(rEarth);


//Methods:
//Natural logs more useful than base 10 logs - Eg. Formal soln module: 
// Fundamental constants
var logC = Math.log(c);
var logSigma = Math.log(sigma);
var logRadConst = Math.log(radConst);  //radiation constant
var logWien = Math.log(wien);
var logKBoltz = Math.log(kBoltz);
var logH = Math.log(h);
var logEe = Math.log(ee); //Named so won't clash with log_10(e)
var logMe = Math.log(mE);
var logGConst = Math.log(GConst);
var logRConst = Math.log(RConst);
//Conversion factors
var logAmu = Math.log(amu);
var logEv = Math.log(eV);
var logRSun = Math.log(rSun);
var logMSun = Math.log(mSun);
var logLSun = Math.log(lSun);
var logAu = Math.log(au);

//Links required to create elements easily
                      //JB
    var xmlns = "http://www.w3.org/2000/xmlns/";
    var xmlnsLink = "xmlns:xlink";
    var xmlnsLink2 = "http://w3.org/1999/xlink"; 
    var xmlW3 = "http://www.w3.org/2000/svg";    

var tiny = 1.0e-49;
var logTiny = Math.log(tiny);


                      //JB
                      //

var lineColor = "#000000"; //black
var lineThick = 1;
var xTickPosCnvs = 0;
var yTickPosCnvs = 0;
var xShiftCnvs = 0;
var yShiftCnvs = 0;
var xPhys = 0;
var yPhys = 0;

//function to output physical data coordinates at clicked cursor location
var dataCoords = function(eventObj, cnvsId,
                          xAxisLength, minXData, rangeXData, xAxisXCnvs,
                          yAxisLength, minYData, rangeYData, yAxisYCnvs){

   //console.log("eventObj.offsetX " + eventObj.offsetX + " eventObj.offsetY " + eventObj.offsetY);
   //console.log("eventObj.pageX " + eventObj.pageX + " eventObj.pageY " + eventObj.pageY);
   //console.log("eventObj.clientX " + eventObj.clientX + " eventObj.clientY " + eventObj.clientY);
   //console.log("eventObj.screenX " + eventObj.screenX + " eventObj.screenY " + eventObj.screenY);
   //var offsetStr = String(eventObj.pageX) + " " + String(eventObj.pageY); 
   //var offsetStr = String(eventObj.clientX) + " " + String(eventObj.clientY); 

//First, erase the previous dataCoords output so output is not clobbered:
   var coordElId;
   if (document.getElementById("coordId") != null){
      //cnvsId.getElementsById("coordId").innerHTML="";
      //console.log("Removing...");
      coordElId = document.getElementById("coordId");
      coordElId.innerHTML=" ";
      coordElId.parentNode.removeChild(coordElId);
   }
   xShiftCnvs = eventObj.offsetX;
   xTickPosCnvs = xShiftCnvs - xAxisXCnvs;
   xPhys = (xTickPosCnvs * rangeXData / xAxisLength) + minXData;
   xPhys = xPhys.toFixed(2);
   yShiftCnvs = eventObj.offsetY;
   yTickPosCnvs = (yAxisYCnvs + yAxisLength) - yShiftCnvs
   yPhys = (yTickPosCnvs * rangeYData / yAxisLength) + minYData;
   yPhys = yPhys.toFixed(2);
   //var offsetStr = String(eventObj.offsetX) + " " + String(eventObj.offsetY); 
   var offsetStr = "<tspan id='coordId' style='font-size:small'>x,y: " + String(xPhys) + " " + String(yPhys) + "</tspan>";
   //console.log("offsetStr " + offsetStr);
   //txtPrint(offsetStr, eventObj.pageX, eventObj.pageY, lineColor, cnvsId);
   //txtPrint(offsetStr, eventObj.clientX, eventObj.clientY, lineColor, cnvsId);
   //txtPrint(offsetStr, eventObj.offsetX, eventObj.offsetY, lineColor, cnvsId);
   //txtPrint(offsetStr, 100, 100, lineColor, cnvsId);
   //coordBoxId.innerHTML = offsetStr;

   return offsetStr;

};



// ********************************************

//***************************  Main ******************************



function main() {



//**********************************************************




// Input control:


    // Get the checkbox values controlling what's plotted:

    // Button for re-computing everything - if stellar parameters have changed
    var btnId = document.getElementById("btnId");
    btnId.onClick = function() {
    };

	//
    //default initializations:

//JQuery:  Independent of order of switches in HTML file?
// Stellar atmospheric parameters
    var numInputs = 7;
//Make settingsId object array by hand:
// setId() is an object constructor
    function setId(nameIn, valueIn) {
        this.name = nameIn;
        this.value = valueIn;
    }

	
	//
    // settingId will be an array of objects
    var settingsId = [];
    settingsId.length = numInputs;
    var flagArr = [];
    flagArr.length = numInputs;
    flagArr[0] = false;
 

    //
    //1st version of each is of JQuery-ui round sliders not available
    // jquery-ui round sliders -->
    //Round sliders Copyright (c) 2015-2016, Soundar
    //      http://roundsliderui.com/
      
    var massStar = 1.0 * $("#starMass").val(); // solar masses
    var radius = 1.0 * $("#radius").val(); // K
   // var index = 1.0 * $("#index").val(); // log cgs
   // var beta = 1.0 * $("#beta").val(); // log cgs
    var Yfrac = 1.0 * $("#Yfrac").val(); // linear factor
    var Zfrac = 1.0 * $("#Zfrac").val(); // linear factor
// Planetary parameters for habitable zone calculation
    var greenHouse = 1.0 * $("#GHTemp").val(); // Delta T_Surf boost K
    //var greenHouse = 1.0 * $("#GHTemp").roundSlider("getValue");
    var atmosPress = 1.0 * $("#AtmPress").val();
    //var atmosPressObj = $("#AtmPress").data("roundSlider");
    //var atmosPress = 1.0 * atmosPressObj.getValue();
    //var greenHouseObj = $("#GHTemp").data("roundSlider");
    //var greenHouse = 1.0 * greenHouseObj.getValue();
    var albedo = 1.0 * $("#Albedo").val(); //unitless reflectivity
    //var albedo = 1.0 * $("#Albedo").roundSlider("getValue");
    //var albedoObj = $("#Albedo").data("roundSlider");
    //var albedo = 1.0 * albedoObj.getValue();
// 

    settingsId[0] = new setId("<em>M</em>", massStar);
    settingsId[1] = new setId("Radius", radius);
   // settingsId[2] = new setId("index", index);
   // settingsId[3] = new setId("beta", beta);
    settingsId[2] = new setId("Yfrac", Yfrac);
    settingsId[3] = new setId("Zfrac", Zfrac);
    settingsId[4] = new setId("<span style='color:green'>AtmP</span>", atmosPress);
    settingsId[5] = new setId("<span style='color:green'>GHEff</span>", greenHouse);
    settingsId[6] = new setId("<span style='color:green'><em>A</em></span>", albedo);

    var solvent = "water"; //default intialization
    //
    //var numPerfModes = 8;
    //var switchPerf = "Fast"; //default initialization
    //JQuery:
    //Fast and Real modes are raio switches - mutuallye xclusive:
//default initializations:
    //
    var ifShowStruc = true;
    var ifStandardSun = true;
    //
    var ifPrintNone = true;
    var ifPrintStruc = false;
    //

//    // Display options:
//    if ($("#showStruc").is(":checked")) {
//        ifShowStruc = true; // checkbox
//    }

    //Detailed print-out options:
    if ($("#printNone").is(":checked")) {
        ifPrintNone = true; // checkbox
    }
    if ($("#printStruc").is(":checked")) {
        ifPrintStruc = true; // checkbox
    }
console.log("ifPrintStruc " + ifPrintStruc);
    var ifPrintStruc = true;
    //       
//
    var switchStar = "None";
    var numPreStars = 9;
    //JQuery:
    // None: (default)
    if ($("#none").is(":checked")) {
        switchStar = $("#none").val(); // radio 
    }

// Sun
    if ($("#sun").is(":checked")) {
        switchStar = $("#sun").val(); // radio 
    }
// Vega
    if ($("#vega").is(":checked")) {
        switchStar = $("#vega").val(); // radio 
    }
// Arcturus
    if ($("#arcturus").is(":checked")) {
        switchStar = $("#arcturus").val(); // radio 
    }

// Proxima Centauri (lburns)
    if ($("#alphacentc").is(":checked")) {
	switchStar = $("#alphacentc").val(); // radio
    }

//JQuery:
    if (switchStar === "Sun") {
        var massStar = 1.0;
        settingsId[0].value = 1.0;
        var radius = 1.0;
        settingsId[1].value = 1.0;
       // var index = 3.0;
       // settingsId[2].value = 3.0;
       // var beta = 0.9;
       // settingsId[3].value = 0.9;
        var Yfrac = 0.28;
        settingsId[2].value = 0.28;
        var Zfrac = 0.02;
        settingsId[3].value = 0.02;
    }

    if (switchStar === "Arcturus") {
        var massStar = 0.5;
        settingsId[0].value = 0.5;
        var radius = 20.0;
        settingsId[1].value = 20.0;
       // var index = 1.5;
       // settingsId[2].value = 1.5;
       // var beta = 0.9;
       // settingsId[3].value = 0.9;
        var Yfrac = 0.29;
        settingsId[2].value = 0.29;
        var Zfrac = 0.01;
        settingsId[3].value = 0.01;
    }

    if (switchStar === "Vega") {
        var massStar = 2.0;
        settingsId[0].value = 2.0;
        var radius = 2.0;
        settingsId[1].value = 2.0;
       // var index = 3.0;
       // settingsId[2].value = 3.0;
       // var beta = 0.5;
       // settingsId[3].value = 0.5;
        var Yfrac = 0.28;
        settingsId[2].value = 0.28;
        var Zfrac = 0.02;
        settingsId[3].value = 0.02;
    }


    var switchSolvent = "Water";
    var numSolvents = 4;

// Water 
    if ($("#water").is(":checked")) {
        switchSolvent = $("#water").val(); // radio 
        solvent = "water";
    }
// Methane 
    if ($("#methane").is(":checked")) {
        switchSolvent = $("#methane").val(); // radio 
        solvent = "methane";
    }
// Ammonia 
    if ($("#ammonia").is(":checked")) {
        switchSolvent = $("#ammonia").val(); // radio 
        solvent = "ammonia";
    }
// Carbon dioxide 
    if ($("#carbonDioxide").is(":checked")) {
        switchSolvent = $("#carbonDioxide").val(); // radio 
        solvent = "carbonDioxide";
    }
   

    var switchPlanet = "None";
    var numPrePlanets = 4;
    //JQuery:
    // None: (default)
    if ($("#noneplanet").is(":checked")) {
        switchPlanet = $("#noneplanet").val(); // radio 
    }

// Earth
    if ($("#earth").is(":checked")) {
        switchPlanet = $("#earth").val(); // radio 
    }

    if (switchPlanet === "Earth") {
        var solvent = "water";
        $('input[name="switchSolvent"]').val(["water"]);
        atmosPress = 101.3;
        var AtmPress = 101.3;
        settingsId[4].value = 101.3;
        //$("#AtmPress").val(101.3);
        //$("#AtmPress").roundSlider("setValue", "100.0");
        greenHouse = 33.0;
        var GHTemp = 33.0;
        settingsId[5].value = 33.0;
        //$("#GHTemp").val(33.0);
        //$("#GHTemp").roundSlider("setValue", "33.0");
        var albedo = 0.3;
        var Albedo = 0.3;
        settingsId[6].value = 0.3;
        //$("#Albedo").val(0.3);
        //$("#Albedo").roundSlider("setValue", "0.3");
    }

// Mars 
    if ($("#mars").is(":checked")) {
        switchPlanet = $("#mars").val(); // radio 
    }

    if (switchPlanet === "Mars") {
        var solvent = "water";
        $('input[name="switchSolvent"]').val(["water"]);
        atmosPress = 0.6;
        var AtmPress = 0.6; //kPa
        settingsId[4].value = 0.6;
        //$("#AtmPress").val(0.6);
        //$("#AtmPress").roundSlider("setValue", "0.6");
        greenHouse = 5.0;
        var GHTemp = 5.0;
        settingsId[5].value = 5.0;
        //$("#GHTemp").val(5.0);
        //$("#GHTemp").roundSlider("setValue", "5.0");
        albedo = 0.25;
        var Albedo = 0.25;
        settingsId[6].value = 0.25;
        //$("#Albedo").val(0.25);
        //$("#Albedo").roundSlider("setValue", "0.25");
    }

// Venus 
    if ($("#venus").is(":checked")) {
        switchPlanet = $("#venus").val(); // radio 
    }

    if (switchPlanet === "Venus") {
        solvent = "water";
        $('input[name="switchSolvent"]').val(["water"]);
        atmosPress = 9300.0;
        var AtmPress = 9300.0;
        settingsId[4].value = 9300;
        //$("#AtmPress").val(9300);
        //$("#AtmPress").roundSlider("setValue", "9300");
        greenHouse = 510.0;
        var GHTemp = 510.0;
        settingsId[5].value = 510.0;
        //$("#GHTemp").val(510.0);
        //$("#GHTemp").roundSlider("setValue", "510.0");
        albedo = 0.75;
        var Albedo = 0.75;
        settingsId[6].value = 0.75;
        //$("#Albedo").val(0.75);
        //$("#Albedo").roundSlider("setValue", "0.75");
    }

// Titan
    if ($("#titan").is(":checked")) {
        switchPlanet = $("#titan").val(); // radio 
    }

    if (switchPlanet === "Titan") {
        var solvent = "methane";
        $('input[name="switchSolvent"]').val(["methane"]);
        atmosPress = 145.0;
        var AtmPress = 145.0;
        settingsId[4].value = 145.0;
        //$("#AtmPress").val(145.0);
        //$("#AtmPress").roundSlider("setValue", "145.0");
        greenHouse = 12.0;
        var GHTemp = 12.0; 
        settingsId[5].value = 12.0;
        //$("#GHTemp").val(12.0);
        //$("#GHTemp").roundSlider("setValue", "12.0");
        albedo = 0.75;
        var Albedo = 0.21;
        settingsId[6].value = 0.21;
        //$("#Albedo").val(0.21);
        //$("#Albedo").roundSlider("setValue", "0.21");
    }

//console.log("solvent = " + solvent);


////Detailed checmical composition:
//Abundance table adapted from PHOENIX V. 15 input bash file
// Grevesse Asplund et al 2010
//Solar abundances:
// c='abundances, Anders & Grevesse',

  var numDeps = 5000;
//  var nome = [];


  var logE10 = Math.log(10.0); // convert log_10 to log_e
  var log10E = logTen(Math.E); // convert log_e to log_10
  var logPi = Math.log(Math.PI);

    //
    // Form validation and Initial sanity checks:
    // 

// Stellar parameters:
//
    flagArr[0] = false;
    if (massStar === null || massStar == "") {
        alert("mass must be filled out");
        return;
    }
    flagArr[0] = false;
    if (massStar < 0.2) {
        flagArr[0] = true;
        massStar = 0.2;
        var massStarStr = "0.2";
        settingsId[0].value = 0.2;
    }
    if (massStar > 5.0) {
        flagArr[0] = true;
        massStar = 5.0;
        var massStarStr = "5.0";
        settingsId[0].value = 5.0;
    }
    
    var minRadius = 0.2; //solar radii
    var maxRadius = 5.0 //solar radii
    flagArr[1] = false;
    if (radius === null || radius == "") {
        alert("Radius must be filled out");
        return;
    }
    if (radius < minRadius) {
        flagArr[1] = true;
        radius = minRadius;
        var radiusStr = String(minRadius);
        settingsId[1].value = minRadius;
    }
    if (radius > maxRadius) {
        flagArr[1] = true;
        radius = maxRadius;
        var radiusStr = String(maxRadius);
        settingsId[1].value = maxRadius;
    }
    
//    if (index === null || index === "") {
//        alert("index must be filled out");
//        return;
//    }
//    var minIndex = 0.0; 
//    var maxIndex = 5.0; 
//    var minIndexStr = "0.0";
//    var maxIndexStr = "5.0";
//    flagArr[2] = false;
//    if (index < minIndex) {
//        flagArr[2] = true;
//        index = minIndex;
//        var indexStr = minIndexStr;
//        settingsId[2].value = minIndex;
//    }
//    if (index > maxIndex) {
//        flagArr[2] = true;
//        index = maxIndex;
//        var indexStr = maxIndexStr;
//        settingsId[2].value = maxIndex;
//    }

//    if (beta === null || beta === "") {
//        alert("beta must be filled out");
//        return;
//    }
//    flagArr[3] = false;
//    if (beta < 0.1) {
//        flagArr[3] = true;
//        beta = 0.1;
//        var betaStr = "0.1";
//        settingsId[3].value = 0.1;
//    }
//    if (beta > 0.9) {
//        flagArr[3] = true;
//        beta = 0.9;
//        var betaStr = "0.9";
//        settingsId[3].value = 0.9;
//    }
	
    if (Yfrac === null || Yfrac == "") {
        alert("Yfrac must be filled out");
        return;
    }

    flagArr[2] = false;
    if (Yfrac < 0.1) {
        flagArr[2] = true;
        Yfrac = 0.1;
        var YStr = "0.1";
        settingsId[2].value = 0.1;
    }
    if (Yfrac > 0.5) {
        flagArr[2] = true;
        Yfrac = 0.5;
        var YStr = "0.5";
        settingsId[2].value = 0.5;
    }

    if (Zfrac === null || Zfrac == "") {
        alert("Zfrac must be filled out");
        return;
    }
    flagArr[3] = false;
    if (Zfrac < 0.001) {
        flagArr[3] = true;
        Zfrac = 0.001;
        var ZStr = "0.001";
        settingsId[3].value = 0.001;
    }
    if (Zfrac > 0.2) {
        flagArr[3] = true;
        Zfrac = 0.2;
        var ZStr = "0.2";
        settingsId[3].value = 0.2;
    }

 // Find the Hydrogen mass fraction:
     var Xfrac = 1.000 - Yfrac;
     Xfrac = Xfrac - Zfrac;

  var logRadius = Math.log(radius);
//  console.log("switchStar " +  switchStar + " M " + massStar + " R " + radius + " Yfrac " + Yfrac + " Zfrac " + Zfrac); 

//Mean molecular weight of specified mixture in amu
// - assumes complete ionization of all species:
   var muIamu = muI(Xfrac, Yfrac, Zfrac); 
//   console.log("muIamu (amu) " + muIamu);
   var muIval = muIamu * amu; // a.m.u. to g
   var logMuIamu = Math.log(muIamu);  // amu 
   var logMuIval = Math.log(muIval);  // cgs units

// Eddington standard model:
// Ideal gas + photon mixture:
// Polytropic index, "n":

   var index = 3.0E0;

//Fore reference - the polytropic exponent, gamma, corresponding to the
//polytropic index:

   var gamma = 1.0 + (1.0/index);
//   console.log("gamma " + gamma);

// Find the *intial* logaritmic surface gravity, log(g) in cgs units:
// This will have to refined alter when we find the actual radius of the model

   var logMassCGS = Math.log(massStar) + logMSun;
   var logRadiusCGS = Math.log(radius) + logRSun;
   var logg = logGConst + logMassCGS - 2.0*logRadiusCGS;
   var logg10 = log10E*logg;
   var massCGS = Math.exp(logMassCGS);
   var radiusCGS = Math.exp(logRadiusCGS);

// Evaluate mean mass density:
   var logRhoBar = Math.log(3.0) + logMassCGS - Math.log(4.0) - logPi - 3.0*logRadiusCGS;  
   var rhoBar = Math.exp(logRhoBar);

	// Planetary parameters for habitable zone calculation:
    //
    
        var earthP = 101.3; //Earth sea level pressure in kPa
//initialize variables with values for water:
       var phaseA = 4.6543;
       var phaseB = 1435.264;
       var phaseC = -64.848; 
       var criticalTemp = 647.096; //K 
       var criticalPress = 22.06e3; //kPa 
       var criticalPressStr = "22060"; //kPa 
       var tripleTemp = 273.16; //K 
       var triplePress = 0.611657; //kPa 
       var triplePressStr = "0.62"; //kPa 
//Solvent blocks with Antoine coefficients (A, B, C) for approximate relation between 
//temperature in K and vapor pressure bar 
// and critical- and triple- point temperatures and pressures
//- taken from NIST Chemistry WebBook
//Brown, R.L. & Stein, S.E., "Boiling Point Data" in NIST Chemistry WebBook, NIST Standard Reference Database Number 69, Eds. P.J. Linstrom and W.G. Mallard, National Institute of Standards and Technology, Gaithersburg MD, 20899, doi:10.18434/T4D303, (retrieved April 12, 2017).
   if (solvent == "water"){
//  Values for T below 100 C = 373 K 
       if (atmosPress <= earthP){
         phaseA = 4.6543;
         phaseB = 1435.264;
         phaseC = -64.848; 
       } else {
         phaseA = 3.55959;
         phaseB = 643.748;
         phaseC = -198.043;
       }
       criticalTemp = 647.096; //K 
       criticalPress = 22.06e3; //kPa 
       criticalPressStr = "22060"; //kPa 
       tripleTemp = 273.16; //K 
       triplePress = 0.611657; //kPa 
       triplePressStr = "0.62"; //kPa 
   } //water block
   if (solvent == "methane"){
       phaseA = 3.9895;
       phaseB = 443.028;
       phaseC = -0.49; 
       criticalTemp = 190.8; //K 
       criticalPress = 4640.0; //kPa 
       criticalPressStr = "4640"; //kPa 
       tripleTemp = 90.68; //K 
       triplePress = 11.7; //kPa 
       triplePressStr = "12"; //kPa 
   } //methane block
   if (solvent == "ammonia"){
//  Values for T below ??? K 
      // if (atmosPress <= ????){
         phaseA = 3.18757;
         phaseB = 506.713;
         phaseC = -80.78; 
      // } else {
      //   phaseA = 4.86886;
      //   phaseB = 1113.928;
      //   phaseC = -10.409;
      // }
       criticalTemp = 405.5; //K 
       criticalPress = 11280; //kPa 
       criticalPressStr = "11280"; //kPa 
       tripleTemp = 195.40; //K 
       triplePress = 6.076; //kPa 
       triplePressStr = "6.1"; //kPa 
   } //ammonia block
   if (solvent == "carbonDioxide"){
       phaseA = 6.81228;
       phaseB = 1301.679;
       phaseC = -3.494; 
       criticalTemp = 304.19; //K 
       criticalPress = 7380; //kPa 
       criticalPressStr = "7380"; //kPa 
       tripleTemp = 216.55; //K 
       triplePress = 517.0; //kPa 
       triplePressStr = "517"; //kPa 
   } //carbonDioxide block
  
    if (atmosPress === null || atmosPress === "") {
        alert("atmosPress must be filled out");
        return;
    }
    flagArr[4] = false;
    if (atmosPress < triplePress) {
        flagArr[4] = true;
        atmosPress = triplePress;
        var atmosPressStr = triplePressStr;
        settingsId[4].value = triplePress;
        //$("#AtmPress").val(triplePress);
        //$("#AtmPress").roundSlider("setValue", triplePress);
    }
    if (atmosPress > criticalPress) {
        flagArr[4] = true;
        atmosPress = criticalPress;
        var atmosPressStr = criticalPressStr;
        settingsId[4].value = criticalPress;
        //$("#AtmPress").val(criticalPress);
        //$("#AtmPress").roundSlider("setValue", criticalPress);
    }
    flagArr[5] = false;
    if (greenHouse < -200.0) {
        flagArr[5] = true;
        greenHouse = -200.0;
        var greenHouseStr = "-200.0";
        settingsId[5].value = -200.0;
        //$("#GHTemp").val(-200.0);
        //$("#GHTemp").roundSlider("setValue", -200.0);
    }
    if (greenHouse > 600.0) {
        flagArr[5] = true;
        greenHouse = 600.0;
        var greenHouseStr = "600.0";
        settingsId[5].value = 600.0;
        //$("#GHTemp").val(600.0);
        //$("#GHTemp").roundSlider("setValue", 600.0);
    }
    if (albedo === null || albedo === "") {
        alert("albedo must be filled out");
        return;
    }
    flagArr[6] = false;
    if (albedo < 0.0) {
        flagArr[6] = true;
        albedo = 0.0;
        var albedoStr = "0.0";
        settingsId[6].value = 0.0;
        //$("#Albedo").val(0.0);
        //$("#Albedo").roundSlider("setValue", 0.0);
    }
    if (albedo > 0.95) {
        flagArr[6] = true;
        albedo = 0.95;
        var albedoStr = "0.95";
        settingsId[6].value = 0.95;
        //$("#Albedo").val(0.95);
        //$("#Albedo").roundSlider("setValue", 0.95);
    }


// This has to be up here for some reason:
// Get the ID of the container div:


    var textId = document.getElementById("outPanel"); // text output area

//Input and output divs:
    var starInputId = document.getElementById("StarInputs");
    var starOutputId = document.getElementById("StarOutputs");
    var planetInputId = document.getElementById("PlanetInputs");
//    var planetOutputId = document.getElementById("PlanetOutputs");

    var plotOneId = document.getElementById("plotOne");
    var cnvsOneId = document.getElementById("plotOneCnvs");
    var plotTwoId = document.getElementById("plotTwo");
    var cnvsTwoId = document.getElementById("plotTwoCnvs");
    var plotThreeId = document.getElementById("plotThree");
    var cnvsThreeId = document.getElementById("plotThreeCnvs");
    var plotFourId = document.getElementById("plotFour");
    var cnvsFourId = document.getElementById("plotFourCnvs");
    var plotFiveId = document.getElementById("plotFive");
    var cnvsFiveId = document.getElementById("plotFiveCnvs");
    var plotSevenId = document.getElementById("plotSeven");
    var cnvsSevenId = document.getElementById("plotSevenCnvs");
    var plotNineId = document.getElementById("plotNine");
    var cnvsNineId = document.getElementById("plotNineCnvs");
    var plotTenId = document.getElementById("plotTen");
    var cnvsTenId = document.getElementById("plotTenCnvs");
    var plotElevenId = document.getElementById("plotEleven");
    var cnvsElevenId = document.getElementById("plotElevenCnvs");
    var plotTwelveId = document.getElementById("plotTwelve");
    var cnvsTwelveId = document.getElementById("plotTwelveCnvs");
    var plotFourteenId = document.getElementById("plotFourteen");
    var cnvsFourteenId = document.getElementById("plotFourteenCnvs");

    var printModelId = document.getElementById("printModel"); //detailed model print-out area


    if (ifShowStruc === true) {
        plotOneId.style.display = "block";
        plotTwoId.style.display = "block";
        plotThreeId.style.display = "block";
        plotFourteenId.style.display = "block";
    }

    if (ifShowStruc === false) {
        plotOneId.style.display = "none";
        plotTwoId.style.display = "none";
        plotThreeId.style.display = "none";
        plotFourteenId.style.display = "none";
    }

    if (ifPrintStruc === true) { 
        printModelId.style.display = "block";
    } else if (ifPrintNone === true) {
        printModelId.style.display = "none";
    }
console.log("ifPrintStruc " + ifPrintStruc);

    // Begin compute code:


    //Atmospheric structure and Voigt line code code begins here:
    // Initial set-up:

    //var numLams = 250;
    var numLams = 200;
    //var lamUV = 300.0;
    var lamUV = 260.0;
    //var lamIR = 1000.0;
    var lamIR = 2600.0;
    var lamSetup = [lamUV * 1.0e-7, lamIR * 1.0e-7, numLams]; //Start, end wavelength (nm), number of lambdas
 
    var lambdaScale = lamgrid(numLams, lamSetup); //nm


    // Solar parameters:
    var teffSun = 5778.0;
    var loggSun = 4.44;
    var gravSun = Math.pow(10.0, loggSun);

    //Solar units:
    var massSun = 1.0;
    var radiusSun = 1.0;
    var rhoCSun = 1.60E2;  //central density CGS
    var rhoBarSun = 1.41; // mean density CGS
    var pTotCSun = 2.0E17; //central total pressure cgs
    var tempCSun = 1.57E7; //central temperature (K)
    //var radius = Math.sqrt(massStar * gravSun / grav); // solar radii

//   console.log("logg " + logg10 + " rhoBar " + rhoBar + " rhoBarSun " + rhoBarSun);

//Stellar structure:
//
// initializations:
    var temp = [];
    temp.length = numDeps;
    var logTemp = [];
    logTemp.length = numDeps;
    var rvar = [];
    rvar.length = numDeps;
    var rvarMm = [];
    rvarMm.length = numDeps;
    var logRvar = [];
    logRvar.length = numDeps;
    var logKappa = [];
    logKappa.length = numDeps;
    var pTot = [];
    pTot.length = numDeps;
    var logPTot = [];
    logPTot.length = numDeps;
    var pGas = [];
    pGas.length = numDeps;
    var logPGas = [];
    logPGas.length = numDeps;
    var pRad = [];
    pRad.length = numDeps;
    var logPRad = [];
    logPRad.length = numDeps;
    var rho = [];
    rho.length = numDeps;
    var logRho = [];
    logRho.length = numDeps;
// cumulative interior mass variable, m:
    var mvar = [];
    mvar.length = numDeps;
    var logMvar = [];
    logMvar.length = numDeps;
// cumulative interior mass variable, m_r:
    var mvar = [];
    mvar.length = numDeps;
    var logMvar = [];
    logMvar.length = numDeps;
// shell mass, dm_r - needed to calculate shell luminosity 
    var mShell = [];
    mShell.length = numDeps;
// cumulative interior luminosity , L_r:
    var lvar= [];
    lvar.length = numDeps;
    var logLvar = [];
    logLvar.length = numDeps;
// shell luminosity 
    var lShell = [];
    lShell.length = numDeps;
    var lShellPP = [];
    lShellPP.length = numDeps;
    var lShellCNO = [];
    lShellCNO.length = numDeps;
    var epsPP = [];
    epsPP.length = numDeps;
    var epsCNO = [];
    epsCNO.length = numDeps;
//Frequency-averaged mean extinction (opacity):
    var kappa = [];
    kappa.length = numDeps;
    var kapBF = [];
    kapBF.length = numDeps;
    var kapFF = [];
    kapFF.length = numDeps;
    var kapEs = [];
    kapEs.length = numDeps;
    var kapHmin = [];
    kapHmin.length = numDeps;


////Initial value of polytropic constant K for Eddington standard model
//// - mixture of ideal gas and photons
////
  var beta0 = 0.9;  // mean Pgas/Ptot guess for Sun
  var logKfac1 = Math.log(3.0) + 4.0*logRConst - logRadConst - 4.0*logMuIamu;
//	console.log("logMuIamu " + log10E*logMuIamu + " logKfac1 " + log10E*logKfac1);
  logKfac1 = 0.3333 * logKfac1;
	//console.log(" logKfac1 " + log10E*logKfac1);
  var betaComp0 = 1.0E0 - beta0;
  var logKfac2 = Math.log(betaComp0) - 4.0*Math.log(beta0);
  logKfac2 = 0.3333 * logKfac2;
	//console.log("logKfac2 " + log10E*logKfac2);
  var logK0 = logKfac1 + logKfac2;
  var K0 = Math.exp(logK0);
//  console.log("K0 guess " + K0);
  

// Lame-Emden (L-E) variables
// Using notation of Stellar Structure and Evolution, 2nd Ed.m
// Kippenhahn, Weigert & Weiss, Ch. 19

// L-E independent depth variable z:
// Establish arbitrary z grid:
var deltaZ = 0.01E0;   //Guess
var logDeltaZ = Math.log(deltaZ);
var z = [];
z.length = numDeps;
var logZ = [];
logZ.length = numDeps;
z[0] = deltaZ; //Offset by one Delta from origin at centre of star to avoid numerical problems
logZ[0] = Math.log(z[0]);
// L-E dependent variable w:
var w = [];
w.length = numDeps;
var logW = [];
logW.length = numDeps;


for (var i = 1; i < numDeps; i++){
  z[i] = z[i-1] + deltaZ; 
  logZ[i] = Math.log(z[i]);
//Initialize w:
  w[i] = 1.0;	
  logW[i] = 0.0;
}

// 4th order central region polynomial approximation to seed a multi-step method and for comparison

var w4 = [];
w4.length = numDeps;
//By defn, w at centre of star is 1.0 exactly:
w4[0] = 1.0E0;


for (var i = 1; i < numDeps; i++){
    w4[i] = 1.0 - (z[i]**2)/6.0 + index*(z[i]**4)/120.0;
//    if ((i % 100) == 0){
//       console.log("i " + i + " z " + z[i] + " w4 " + w4[i]);
//    }
}

// Numerical solution of 2nr order ODE, the Lane-Emmed Eq.
//

var iUpper = 0; //initialization
var factor = 0.0;
var logFactor = 0.0;

var firstZero = false;

//By defn, w at centre of star is 1.0 exactly:
w[0] = 1.0E0;
logW[0] = Math.log(w[0]);

/*
//1st order method:

//Initialize the helper function
var helpF = -1.0 * z[0]**2 * (w[0]**index) * deltaZ;
//console.log("helpF " + helpF);

for (var i = 1; i < numDeps; i++){
    //logFactor = Math.log(helpF) + logDeltaZ - 2.0*logZ[i]; // will NOT work - ive argument
    //factor = Math.exp(logFactor);
    factor = helpF * deltaZ / (z[i]*z[i]);
    w[i] = w[i-1] + factor;
    helpF = helpF - z[i]**2 * (w[i]**index) * deltaZ;
// Find log(W) is safe and and Find upper boundary:
    if ( w[i] <= 0.0 ){
       firstZero = true;
    }
    if ( (w[i] > 0.0) && (firstZero == false) ){
       logW[i] = Math.log(w[i]);
       iUpper++;
       //if ((i % 100) == 0){
       //   console.log("i " + i + " z " + z[i] + " w4 " + w4[i] + " w " + w[i] + " logW " + logW[i]);
       //}
    } 
}
*/

//Trapezoid rule:
//solution at 2nd grid point is from Yarlor series expansion about centre:
w[1] = w4[1];
logW[1] = Math.log(w[1]);
w[2] = w4[2];
logW[2] = Math.log(w[2]);

//First two values of the helper function:
var f0 = -0.5 * ( (z[0]**2 * (w[0]**index)) 
                 +   (z[1]**2 * (w[1]**index)) ) * deltaZ;
var deltaF =  -0.5 * ( (z[1]**2 * (w[1]**index))
                 +   (z[2]**2 * (w[2]**index)) ) * deltaZ;
var f1 = f0 + deltaF;

for (var i = 3; i < numDeps; i++){
   factor = 0.5 * ( (f0/(z[i-1]*z[i-1]))
	        + (f1/(z[i]*z[i])) ) * deltaZ;
   w[i] = w[i-1] + factor;
//Update helper function, which is also an intergral
   deltaF = -0.5 * ( (z[i-2]**2 * (w[i-2]**index))
             +   (z[i-1]**2 * (w[i-1]**index)) ) * deltaZ;
   f0 = f0 + deltaF;
   deltaF =  -0.5 * ( (z[i-1]**2 * (w[i-1]**index))
                 +   (z[i]**2 * (w[i]**index)) ) * deltaZ;
   f1 = f0 + deltaF;
// Find log(W) is safe and and Find upper boundary:
    if ( w[i] <= 0.0 ){
       firstZero = true;
    }
    if ( (w[i] > 0.0) && (firstZero == false) ){
       logW[i] = Math.log(w[i]);
       iUpper++;
    }
}


//Upper boundary:
  var zn = z[iUpper];
//  console.log("iUpper " + iUpper + " zn " + zn);


// L-E hlper variable, A in notation of Kippenhahn, Weigert & Weiss:
  var logA = Math.log(1.0E0*zn) - logRadiusCGS;
  var A = Math.exp(logA);
//  console.log("A = " + A);

// Evaluate dw/dz|zn gradient:
// 1st order for now!
// Fon now:  Fudge upper gradient to get a model that's close to teh Sun's outputs with the SUn's inputs::
  var gradFudge = 0.5;
//  var gradFudge = 1.0/Math.PI;  //desparation

// Tale mean of several 1st order granients near upper boundary:
//
  var numSamp = 10;
  var dwBydzn = [];
  dwBydzn.length = numSamp;
  var totDwBydzn = 0; // initialization:
  var j, k;
  for (var i = 0; i < numSamp; i++){
    j = i+1;
    k = i+2;
    dwBydzn[i] = (w[iUpper-j] - w[iUpper-k]) / (z[iUpper-j] - z[iUpper-k]);  
    totDwBydzn += dwBydzn[i];
  }

  var meanDwBydzn = totDwBydzn / numSamp;

//  var dwBydzn = (w[iUpper] - w[iUpper-1]) / (zn - z[iUpper-1]);
  var meanDwBydzn = gradFudge * meanDwBydzn;

// dw/dz is -ve - find its absolute value: // not true?? It shoudl be!!
  var absdwBydzn = Math.abs(meanDwBydzn);
//  console.log("absdwBydzn = " + absdwBydzn);

//Evaluate central density:
   var logRhoC = Math.log(0.3333E0) + Math.log(zn) + logRhoBar - Math.log(absdwBydzn); 
   var rhoC = Math.exp(logRhoC);

//// !!!  Temporary fix to test forced solar solution:
//  rhoC = 160.0;  //tempraray to force solar solution!!
//  logRhoC = Math.log(rhoC);
//// !!!!!!

//  console.log("rhoC = " + rhoC + " rhoCSun " + rhoCSun);
//post hoc value of polytropic constant, K
// - compare to K0 from Eddington standard model
   var logK = Math.log(4.0) + logPi + logGConst + (index-1.0)*logRhoC/index - Math.log(index+1.0) - 2.0*logA; 
   var K = Math.exp(logK);
  // console.log("K = " + K);

   var dwBydz = 0.0;  //initialization


//Center of star (central boundary, r=0 & z=0:
  rvar[0] = 0.0;
  rvarMm[0] = 0.0;
  logRvar[0] = tiny;
  logRho[0] = logRhoC + index*logW[0];
  rho[0] = Math.exp(logRho[0]);
  logPTot[0] = logK + (index+1.0)*logRho[0]/index;
  pTot[0] = Math.exp(logPTot[0]);
//  console.log("i 0 " + " rho " + rho[0] + " logPTot " +  log10E*logPTot[0]);
//  console.log("Sun rhoC " + rhoCSun + " pTotC " + log10E*Math.log(pTotCSun));
  var absdwBydz = 0.0;

// Compute mechanical structure 
   for (var i = 1; i < iUpper; i++){
      logRvar[i] = Math.log(z[i]) - logA;
      rvar[i] = Math.exp(logRvar[i]);
      rvarMm[i] = 1.0e-8 * rvar[i];
      logRho[i] = logRhoC + index*logW[i]; 
      rho[i] = Math.exp(logRho[i]);
      logPTot[i] = logK + (index+1.0)*logRho[i]/index; 
      pTot[i] = Math.exp(logPTot[i]); 
// local dw/dz gradient
      dwBydz = ( w[i] - w[i-1]) / (z[i] - z[i-1] );
// Apply same fudge factor to gradient needed to get agreement with solar model:
      dwBydz = gradFudge * dwBydz;
  //dwBydz is -ve - find absolute value:
      absdwBydz = Math.abs(dwBydz); 
      //logMvar[i] = Math.log(4.0) + logPi + logRhoC + 3.0*logRvar[i] + Math.log(absdwBydz) - logZ[i];
      //mvar[i] = Math.exp(logMvar[i]);
      mvar[i] = 4.0 * Math.PI * rhoC * (rvar[i]**3.0) * dwBydz / ( -1.0 * z[i] ); 
      logMvar[i] = Math.log(mvar[i]);
      mShell[i] =  mvar[i] -  mvar[i-1];
      //if ((i % 100) == 0){
      //   console.log("i " + i + " logR " + log10E*logRvar[i] + " logRho " + log10E*logRho[i] + " logPTot " +  log10E*logPTot[i]
      //      + " logMr " + log10E*logMvar[i]);
      //}
   }


//Recover actual value of beta from value of K
// 4th order polynomial with beta as independent variable:
  var logA4 = 3.0*logK + logRadConst + 4.0*logMuIamu - Math.log(3.0) - 4.0*logRConst;
  var a4 = Math.exp(logA4);
 //console.log("logA4 " + log10E*logA4);
// Note: a3 = a2 = 0
  var a1 = 1.0E0;
  var a0 = -1.0E0;
//Coefficenets of 3rd order derivateive polynomial, df/dBeta:
// Note:  For each case, K is effectively a constant independent of beta - ??
  var b3 = 4.0 * a4; 
//Note b2 = b1 = 0
  var b0 = 1.0E0;
  
  var maxIter = 10;
//beta in range [0,1], so initial guess is 0.5;
  var oldRoot = 0.5;
// initializations:
  var newRoot = oldRoot; 
  var iter = 0;
  var f = 0;
  var fPrime = 0;
// Convergence criterion (range of beta is [0,1]:
  var epsilon = 0.05;
  var change = 1.0;


//Newton's methof to find root for f(beta) - ie. value of beta:
   while ( (change > epsilon) && (iter < maxIter) ){
      f = a4*(oldRoot**4) + a1*oldRoot + a0;
      fPrime = b3*(oldRoot**3) + b0;
 //safety:
      if (fPrime < tiny){
         fPrime = tiny;
      }
      newRoot = oldRoot - (f/fPrime);
      change = Math.abs(oldRoot-newRoot);
      oldRoot = newRoot;
      iter++;
   }
 var beta = newRoot;

// console.log("beta = " + beta + " iter = " + iter);


// Find value of temperature from PTot at each radius:
// Assume strict thermodynamic equilibrium:  T(r) = T_kin(r) = T_Rad(r)

//function, f(T), is 4th order polynomial with T as independent variable
// - df/dT is a 3rd order polynomial in T

  var a4 = radConst / 3.0E0;
  var b3 = 4.0 * a4;
//Note:  a3 = a2 = 0
  var a1Help = RConst / muIamu;

//Initializations
  var a1 = 0.0;
  var a0 = 0.0;
  var b0 = 0.0;
  maxIter = 10;
// Initial guess at center:
//  Estimate T assuming pTot = pGas, then again assunig pTot = pRad
//    and take the higher value:
  var t0Gas = pTot[0]*muIamu / (rho[0]*RConst);
  var t0Rad = 3.0 * pTot[0] / radConst;
  t0Rad = t0Rad**0.25;
  var oldRoot = Math.max(t0Gas, t0Rad);
//console.log("t0Gas " + t0Gas + " t0Rad " + t0Rad + " oldRoot " + oldRoot);
// initializations:
  var newRoot = oldRoot; 
  var iter = 0;
  var f = 0;
  var fPrime = 0;
// Convergence criterion in *logT*:
  var epsilon = Math.log(oldRoot)/100.0;
// console.log("T epsilon " + epsilon);
  var change = 1.0;


// Outer loop over radial coordinate (shell):
  for (i = 0; i < iUpper; i++){ 

// Initial guess is the T of the previosu layer:
     if (i > 0){	  
        oldRoot = temp[i-1];
     }
     // initializations:
     newRoot = oldRoot; 
     var iter = 0;
     var f = 0;
     var fPrime = 0;
// Convergence criterion (range of beta is [0,1]:
     var change = 1.0;

     a1 = rho[i]*a1Help;
     a0 = -1.0 * pTot[i];
     b0 = a1;
//Newton's methof to find root for f(beta) - ie. value of beta:
     while ( (change > epsilon) && (iter < maxIter) ){
        f = a4*(oldRoot**4) + a1*oldRoot + a0;
        fPrime = b3*(oldRoot**3) + b0;
 //safety:
        if (fPrime < tiny){
           fPrime = tiny;
        }
        newRoot = oldRoot - (f/fPrime);
        change = Math.abs(Math.log(oldRoot/newRoot));
        oldRoot = newRoot;
        iter++;
     }

     temp[i] = newRoot;
     logTemp[i] = Math.log(temp[i]);

     logPGas[i] = logRho[i] + logRConst + logTemp[i] - logMuIamu;
     pGas[i] = Math.exp(logPGas[i]);
     logPRad[i] = logRadConst + (4.0*logTemp[i]) - Math.log(3.0);
     pRad[i] = Math.exp(logPRad[i]);

//     if ( (i % 100) == 0.0 ){
//         console.log("i " + i + " temp[i] = " + temp[i] + " iter = " + iter);
//     } 
  } // end outer loop over depth

//  console.log("Central temp: Tc: " + log10E*Math.log(temp[0]) + " Solar Tc " + log10E*Math.log(tempCSun));

//  for (var i = 0; i < iUpper; i++){
//    console.log("i " + i + " temp " + temp[i]);
//  }

// cumulative luminosity is zero at centre of star:
  lvar[0] = 0.0;
  logLvar[0] = logTiny;
//  console.log("lvar[0] " + lvar[0]);
 
 var deltaR = 0.0;
 var logLShell = 0.0; //helper
 var logCommonFac = 0.0;

//Center of star:
//Approximate mean opacity from Kramer's law
 kapBF[0] = kappaBfFn(temp[0], rho[0], Xfrac, Zfrac); //b-f photo-ionization
 kapFF[0] = kappaFfFn(temp[0], rho[0], Xfrac, Zfrac); //f-f Bremsstrahlung
 kapEs[0] = kappaEsFn(temp[0], rho[0], Xfrac, Zfrac); //Thomson e^- scattering
 kapHmin[0] = kappaHminFn(temp[0], rho[0], Xfrac, Zfrac); //H^- b-f
 kappa[0] = kapBF[0] + kapFF[0] + kapEs[0] + kapHmin[0];

//console.log("Central:  kapBf " + kapBF[0] + " kapFf " + kapFF[0] + " kapEs " + kapEs[0] + " kapHmin " + kapHmin[0] + " kappa[0] " + kappa[0]);
//index of nuclear burning core boundary
var iCore = 1; //initialization 

// Have we extinguished our star?
   var warnInert = false;
   if (temp[1] < fusionPPTemp){
      warnInert = true;
   }

//Central boundary:
   epsPP[0] = 0.0;
   epsCNO[0] = 0.0;
   lShell[0] = 0.0;
//tune nuclear energy gerenation rate to match solar L_bol with solar input M and R:
//Might account for unknown electron screening factors and other QM corrections - ??
//Depth-independent so probably not physical:
   var epsFudge = 0.01;  
// Outer loop over radial coordinate (shell):
//Compute power mass generation rate from nuclear fusion
// AND mean opacity
  //console.log(" Xfrac " +  Xfrac + " Zfrac " + Zfrac + " fusionPPTemp " + fusionPPTemp);
  for (i = 1; i <= iUpper; i++){ 

// re-initialize mass power generation rates each time through loop:

     epsPP[i] = 0.0;
     epsCNO[i] = 0.0;
     lShell[i] = 0.0;

     if (temp[i] >= fusionPPTemp) {
	  iCore = Math.max(1, i);  //update 
          epsPP[i] = ppChain(temp[i], rho[i], Xfrac, Zfrac); //H fusion p-p chain
          epsCNO[i] = cnoCycle(temp[i], rho[i], Xfrac, Zfrac); //H fusion CNO cycle
	  //if (epsPP[i] > 0.0){
       // 	  console.log("i " + i + " Log epsPP " + log10E*Math.log(epsPP[i]));
	//  }
	 // if (epsCNO[i] > 0.0){
       // 	  console.log("i " + i + " Log epsCNO " + log10E*Math.log(epsCNO[i]));
//	  }
          deltaR = rvar[i] - rvar[i-1];
          if (deltaR < tiny){
             deltaR = tiny;  //safety
          }
	  logCommonFac = logRho[i] + Math.log(4.0) + logPi + 2.0*logRvar[i] + Math.log(deltaR);
          logLShell = Math.log(epsPP[i]) + logCommonFac; 
          lShellPP[i] = Math.exp(logLShell); 
          logLShell = Math.log(epsCNO[i]) + logCommonFac; 
          lShellCNO[i] = Math.exp(logLShell); 
//          if ( i < 10 ){
//             console.log("i " + i + " deltaR " + deltaR + " lShellPP[i] " + lShellPP[i] + " lShellCNO[i] " + lShellCNO[i]);
//          }
          lShell[i] = lShellPP[i] + lShellCNO[i];
          lShell[i] = epsFudge * lShell[i];
     } else {
	  lShell[i] = 0.0;  //double safety
     }
     lvar[i] = lvar[i-1] + lShell[i]; 
     if (lvar[i] < tiny){
         lvar[i] = tiny;
     }
     logLvar[i] = Math.log(lvar[i]);
//     if ( i < 10 ){
        //console.log("i " + i + " logLvar[i-1] " + log10E*logLvar[i-1] + " logLvar[i] " + log10E*logLvar[i]);
//     }
	  
//Approximate mean opacity from Kramer's law
    kapBF[i] = kappaBfFn(temp[i], rho[i], Xfrac, Zfrac); //b-f photo-ionization
    kapFF[i] = kappaFfFn(temp[i], rho[i], Xfrac, Zfrac); //f-f Bremsstrahlung
    kapEs[i] = kappaEsFn(temp[i], rho[i], Xfrac, Zfrac); //Thomson e^- scattering
    kapHmin[i] = kappaHminFn(temp[i], rho[i], Xfrac, Zfrac); //H^- b-f
    kappa[i] = kapBF[i] + kapFF[i] + kapEs[i] + kapHmin[i];
    //console.log("i " + i + " kapBf " + kapBF[i] + " kapFf " + kapFF[i] + " kapEs " + kapEs[i] + " kapHmin " + kapHmin[i] + " kappa[i] " + kappa[i]);

  }

  var Lbol = lvar[iUpper-1];  //cgs units
  var logLbol = Math.log(Lbol);
  var bolLum = Lbol / lSun;  //solar luminosities

//Radius of nuclear burning core:
  var rCore = rvar[iCore];
  var logRCore = logRvar[iCore];
  var rCoreFrac = rCore/radiusCGS;
// Mass coordinate of nuclear burning core boundary:
  var mCore = mvar[iCore]; 
  var logMCore = logMvar[iCore];
  var mCoreFrac = mCore/massCGS;

//Take surface temprature to be the outermost T value:
  var logTeff = logLbol - Math.log(4.0) - logPi - 2.0*Math.log(radiusCGS) - logSigma;
  logTeff = 0.25E0 * logTeff;
  var teff = Math.exp(logTeff);

//console.log("logLbol " + log10E*logLbol + " rCore/R " + rCoreFrac + " mCore/M " + mCoreFrac + " T_eff " + teff);  
//console.log("Sun: Lbol = " + log10E*logLSun);


//Compute spectral energy distribution (SED) assuming a Planck function
// with T_Rad = teff
//
////initializations:
  var Blam = [];
  Blam.length = numLams;
  var logBlam = [];
  logBlam.length = numLams;
  var lambda = 0.0;
//
//  var lambda = 0.0;
//  var logBnum = 0.0;
//  var logBexp = 0.0;
//  var Bexp = 0.0;
//  var Bdenom = 0.0;
//  var logB = 0.0;
//
//
//  logBnumHelp = Math.log(2.0) + logH + 2.0*logC;
//  logBexpHelp = logH + logC - logK - Math.log(teff);

  for (i = 0; i < numLams; i++){
    // console.log("lambdaScale[i] " + lambdaScale[i]);
    // logBnum = logBnumHelp - 5.0*Math.log(lambda);
    // logBexp = logBexpHelp - Math.log(lambda);
    // Bexp = Math.exp(logBexp);
    // Bdenom = Math.exp(Bexp) - 1.0;
    // logB[i] = logBnum - Math.log(Bdenom);
     logBlam[i] = planck(teff, lambdaScale[i]);
     Blam[i] = Math.exp(logBlam[i]); 
     //if ( (i % 10) == 0 ){
     //   console.log("i " + i + " logBlam[i] " + log10E*logBlam[i]);
     //}
  }


//// Compute photometrc colors:

    var bandFlux =  UBVRIraw(lambdaScale, logBlam);
    var colors = UBVRI(bandFlux);
//    console.log("colors[0-6]: " + colors[0] + " " + colors[1] + " " + colors[2] + " " + colors[3] + " " + colors[4] + " " + colors[5] + " " + colors[6] + " ");

//    
// Set up the canvas:
//

    // **********  Basic canvas parameters: These are numbers in px - needed for calculations:
    // All plots and other output must fit within this region to be white-washed between runs

    var xRangeText = 2050;
    var yRangeText = 65;
    var xOffsetText = 10;
    var yOffsetText = 10;
    var charToPxText = 4; // width of typical character font in pixels - CAUTION: finesse!

    var zeroInt = 0;
    //these are the corresponding strings ready to be assigned to HTML style attributes


    var xRangeTextStr = numToPxStrng(xRangeText);
    var yRangeTextStr = numToPxStrng(yRangeText);
    var xOffsetTextStr = numToPxStrng(xOffsetText);
    var yOffsetTextStr = numToPxStrng(yOffsetText);
    // Very first thing on each load: White-wash the canvas!!

//Clear the board for latest output:

//First, remove all SVG elements:
    //var listOfSVGNodes = document.querySelectorAll("svg");
//We think everything will be a line or a circle:
    var listOfLineNodes = document.querySelectorAll("line");
    var listOfCircNodes = document.querySelectorAll("circle");
    var listOfTextNodes = document.querySelectorAll("text");

    //var numSVGNodes = listOfSVGNodes.length;
    var numLineNodes = listOfLineNodes.length;
    var numCircNodes = listOfCircNodes.length;
    var numTextNodes = listOfTextNodes.length;

//We have to be prepared that this might be our first time through - ??:
    //if (numSVGNodes > 0){

//Remove line elements (axues, tickmarks, barns, etc.)
       if (numLineNodes > 0){
           for (var iNode = 0; iNode < numLineNodes; iNode++){
               listOfLineNodes[iNode].parentNode.removeChild(listOfLineNodes[iNode]);
           }
       } //numLineNodes > 0 condition

//Remove circle elements (axes, tickmarks, barns, etc.)
       if (numCircNodes > 0){
           for (var iNode = 0; iNode < numCircNodes; iNode++){
               listOfCircNodes[iNode].parentNode.removeChild(listOfCircNodes[iNode]);
           }
       } //numCircNodes > 0 condition

//Remove text elements (Stellar and Planet outputs)
       if (numTextNodes > 0){
           for (var iNode = 0; iNode < numTextNodes; iNode++){
               listOfTextNodes[iNode].parentNode.removeChild(listOfTextNodes[iNode]);
           }
       } //numTextNodes > 

  starOutputId.innerHTML = "";
//  planetOutputId.innerHTML = "";

  starOutputId.innerHTML = "<span class='paneTitle'>Star outputs:</span>";
//  planetOutputId.innerHTML = "<span class='paneTitle'>Planet outputs:</span>";


    var roundNum, remain;
    // R & L_Bol:
    var colr = 0;
    var xTab = 60;

//    console.log("warnInert " + warnInert);
    if (warnInert){
       txtPrintGrid("<span style='color:red'; font:bold;'>WARNING:  INERT OBJECT!!</tspan>", lineColor, starOutputId);
    }

    roundNum = massStar.toPrecision(3);
    txtPrintGrid("<span title='Stellar mass '><em>M</em> = </span> "
            + roundNum
            + " <span title='Solar masses'>\n\
<a href='http://en.wikipedia.org/wiki/Solar_mass' target='_blank'><em>M</em><sub>Sun</sub></a>\n\
</span> ",
            lineColor, starOutputId);


    roundNum = radius.toPrecision(3);
    //console.log("radius " + radius + " roundNum " + roundNum);
    //console.log("colr " + colr + " xTab " + xTab + " lineColor " + lineColor);
    //console.log("textId " + textId);

//    txtPrint("<span title='Stellar radius'><em>R</em> = </span> "
//            + roundNum
//            + " <span title='Solar radii'>\n\
//<a href='http://en.wikipedia.org/wiki/Solar_radius' target='_blank'><em>R</em><sub>Sun</sub></a>\n\
//</span> ",
//            20 + colr * xTab, 15, 200, lineColor, textId);
    txtPrintGrid("<span title='Stellar radius'><em>R</em> = </span> "
            + roundNum
            + " <span title='Solar radii'>\n\
<a href='http://en.wikipedia.org/wiki/Solar_radius' target='_blank'><em>R</em><sub>Sun</sub></a>\n\
</span> ",
            lineColor, starOutputId);

//
  roundNum = (Lbol/lSun).toPrecision(3);
//    txtPrint("<span title='Bolometric luminosity'>\n\
//<a href='http://en.wikipedia.org/wiki/Luminosity' target='_blank'><em>L</em><sub>Bol</sub></a> = \n\
//</span> "
//            + roundNum
//            + " <span title='Solar luminosities'>\n\
//<a href='http://en.wikipedia.org/wiki/Solar_luminosity' target='_blank'><em>L</em><sub>Sun</sub></a>\n\
//</span> ",
//            20 + colr * xTab, 40, 200, lineColor, textId);
    txtPrintGrid("<span title='Bolometric luminosity'>\n\
<a href='http://en.wikipedia.org/wiki/Luminosity' target='_blank'><em>L</em><sub>Bol</sub></a> = \n\
</span> "
            + roundNum
            + " <span title='Solar luminosities'>\n\
<a href='http://en.wikipedia.org/wiki/Solar_luminosity' target='_blank'><em>L</em><sub>Sun</sub></a>\n\
</span> ",
            lineColor, starOutputId);
// 
// T_eff:


  roundNum = teff.toFixed(2);
//    txtPrint("<span title='Equivalent width: A measure of spectral line strength'>\n\
//Spectral line \n\
//<a href='http://en.wikipedia.org/wiki/Equivalent_width' target='_blank'>W<sub><em>&#955</em></sub></a>: \n\
//</span>"
//            + roundNum
//            + " <span title='picometers'>\n\
//<a href='http://en.wikipedia.org/wiki/Picometre' target='_blank'>pm</a>\n\
//</span>",
//            360, 40, 200, lineColor, textId);
    txtPrintGrid("<span title='effective temperature'>\n\
<a href='https://en.wikipedia.org/wiki/Effective_temperature' target='_blank'><em>T</em><sub>eff</sub></a>: \n\
</span>"
            + roundNum, lineColor, starOutputId);
// 
// 
// Surface gravity:


  roundNum = (logg10).toFixed(2);
//    txtPrint("<span title='Equivalent width: A measure of spectral line strength'>\n\
//Spectral line \n\
//<a href='http://en.wikipedia.org/wiki/Equivalent_width' target='_blank'>W<sub><em>&#955</em></sub></a>: \n\
//</span>"
//            + roundNum
//            + " <span title='picometers'>\n\
//<a href='http://en.wikipedia.org/wiki/Picometre' target='_blank'>pm</a>\n\
//</span>",
//            360, 40, 200, lineColor, textId);
    txtPrintGrid("<span title='Log Surface gravity'>\n\
<a href='https://en.wikipedia.org/wiki/Surface_gravity' target='_blank'>log <em>g</em></a>: \n\
</span>"
            + roundNum, lineColor, starOutputId);
// 

    // UBVRI indices
    var xTab = 80;
    var colr = 0;
    var roundNum0 = colors[0].toFixed(2);
    var roundNum1 = colors[1].toFixed(2);
    var roundNum2 = colors[2].toFixed(2);
    var roundNum3 = colors[3].toFixed(2);
    var roundNum4 = colors[4].toFixed(2);
    var roundNum5 = colors[5].toFixed(2);// lburns
    var roundNum6 = colors[6].toFixed(2);//lburns
    txtPrintGrid("<a href='http://en.wikipedia.org/wiki/UBV_photometric_system' title='Johnson-Cousins U-B photometric color index' target='_blank'>\n\
<span style='color:purple'>U</span>-" +
            "<span style='color:blue'>B\n\
</span>\n\
</a>: " + roundNum0
            + " <a href='http://en.wikipedia.org/wiki/UBV_photometric_system' title='Johnson-Cousins B-V photometric color index' target='_blank'>\n\
<span style='color:blue'>B\n\
</span>-" +
            "<span style='color:#00FF88'>V</span></a>: " + roundNum1
            + " <a href='http://en.wikipedia.org/wiki/UBV_photometric_system' title='Johnson-Cousins V-R photometric color index' target='_blank'>\n\
<span style='color:#00FF88'>V\n\
</span>-" +
            "<span style='color:red'>R\n\
</span>\n\
</a>: " + roundNum2
            + " <a href='http://en.wikipedia.org/wiki/UBV_photometric_system' title='Johnson-Cousins V-I photometric color index' target='_blank'>\n\
<span style='color:#00FF88'>V\n\
</span>-" +
            "<span style='color:red'>I\n\
</span>\n\
</a>: " + roundNum3
            + " <a href='http://en.wikipedia.org/wiki/UBV_photometric_system' title='Johnson-Cousins R-I photometric color index' target='_blank'>\n\
<span style='color:red'>R</span>-" +
            "<span style='color:brown'>I\n\
</span>\n\
</a>: " 
       + roundNum4, lineColor, starOutputId);
//Added another txtPrint function to display V-K and J-K. Adjusted spectralLine over to fit in these new colors. lburns 06
    txtPrintGrid("<a href='http://en.wikipedia.org/wiki/UBV_photometric_system' title='Johnson-Cousins V-K photometric color index' target='_blank'>\n\
<span style='color:#00FF88'>V</span>-" +
            "<span style='color:sienna'>K\n\
</span>\n\
</a>: " + roundNum5
            + " <a href='http://en.wikipedia.org/wiki/UBV_photometric_system' title='Johnson-Cousins J-K photometric color index' target='_blank'>\n\
<span style='color:firebrick'>J</span>-" +
            "<span style='color:sienna'>K\n\
</span>\n\
</a>: " + roundNum6, lineColor, starOutputId);


    // Echo back the *actual* input parameters:
    //Add subclass to each spectral class (lburns)
    var spectralClass = " ";

  var spectralType = makeSpectralClass(teff, logg10);
//txtPrint(spectralType, 600, 40, 200, lineColor, textId);
    txtPrintGrid(spectralType, lineColor, starOutputId);
    xTab = 140;
    var outString, fullNum, j;

// Graphical output section:


//  NOTE!!!
//  
//  The remaining lines of code or so are all devoted to the graphical (and textual) output.  
//  Have not been able to spin this stuff off into separare function that can be called from 
//  seperate source files by the HTML code.  This is baaaaad!  :-(
//    

// ***** WARNING: Do NOT rearrange order of plot-wise code-blocks.  Some blocks use variables declared and initialized in
// previous blocks.  If you want to re-arrange the plots in the WWW page, change the row and column number
// indices (plotRow, plotCol) at the beginning of each block

//    
// Set up the canvas:
//

// Coordinate considerations with HTML5 <canvas>:
//Display coordinates for size and location of canvas with respect to browser viewport
// are set with CSS with JavaScript (JS) as in objectID.style.width = "***px" 
// (just like scripting a <div>)
//
//"Model" (ie. canvas) coordinates set in html document with <canvas width="***", ...>
// - The CSS (panlWidth & panelHeight) and HTML "width"s and "height"s have to match 
// for model and display coordinates to be on the same scale
// AND: display (CSS) coordinates are with repect to upper left corner of browser window (viewport)
//   BUT model (canvas) coordinates are with respect to upper left corner of *canvas*!
// So - we have to run two coordinate systems with different origins, but same scale
//   camelCase variables of form ***Cnvs are in canvas coordinates

//
// Offset note: Quantities that are relative offsets with respect to other elements 
// have name of form ***Offset;
// Generally, yOffsets must be *added* to the reference element's y-coordinate AND
// xOffsets must be *subtracted* from the reference element's x-coordinate
//  (origin is always upper left corner whether viewport or canvas)
//

// **********  Basic canvas parameters: These are numbers in px - needed for calculations:
// All plots and other output must fit within this region to be gray-washed between runs


//June 2017 - Graphics converted from <canvas> to scale-invariant <SVG> by Jason Bayer

// **************************
//
// Global graphical output variables:
//
//  Panel variables 
//  - set with HTML/CSS style parameters 
//  - in pixels
//
//
// These are with respect to browser viewport upper left corner 
// (ie. y coordinate increases *downward*) :
//
//
// Whole number indices for row and column number in plot grid:
   var plotRow, plotCol;
//
   var panelWidth, panelHeight;
// 
   //horizontal and vertical intervals between panel origins
   //  - must be greater than panelWidth & panelHeight, respectively
   //  to avoid panel overlap
   var spacingX, spacingY;  //horizontal and vertical intervals between panel origins
//
   panelWidth = 450;
   panelHeight = 350;
//
// The following panel elements are clickable/hoverable HTML elements, so will be handled in CSS
// and so are with respect to the browser viewport:
// These are offsets from the panel origin (specifid by panelX & panelY), and so *seem*
// a lot like canvas coordinates:
//
// NOTE: Custom functions like txtPrint(), numPrint(), and plotPnt() always take these
// viewport oriented coordinates
// 
   var titleOffsetX, titleOffsetY; //main plot title origin
   titleOffsetX = 10;
   titleOffsetY = 20;

    //these are the corresponding strings in pixel units ready to be assigned to HTML/CSS style attributes
   var panelWidthStr = numToPxStrng(panelWidth);
   var panelHeightStr = numToPxStrng(panelHeight);


//Background color of panels - a gray tone will accentuate most colors:
// 24 bit RGB color in hexadecimal notation:
    var wDefaultColor =  "#B0B0B0"; //default value 
    var wDiskColor = wDefaultColor; //needed to finesse background colour of white light image 

    var charToPx = 4; // width of typical character font in pixels - CAUTION: finesse!

//
//
// These global parameters are for the HTML5 <svg> element 
//   - in pixls
//   - with respect to upper left corner of *panel* (not viewport!)
//   Assumes that <canvas> HTML width/height in html doc and CSS JS 
//   canvas width/height are the same for a 1:1 scaling (see main graphical
//   section comments above). 
//
    //******* axis, tick mark, and tick-value properties: 

//Origins of plot axes
   var xAxisXCnvs, xAxisYCnvs; //x-axis
   var yAxisXCnvs, yAxisYCnvs; //y-axis
   xAxisXCnvs = 95; //should be greater than yAxisNameOffset  
   xAxisYCnvs = panelHeight - 65; //should be greater than xAxisNameOffset  
   yAxisXCnvs = xAxisXCnvs; //Have x & y axes meet at common origin
   // yAxisYCnvs must be initialized below...

//
//Lengths of plot axes
   var xAxisLength, yAxisLength;
   xAxisLength = 300; //should be less than (panelWidth - yAxisXCnvs)    
   yAxisLength = 200; //should be less than (panelHeight - xAxisYCnvs)    
   yAxisYCnvs = xAxisYCnvs - yAxisLength; // *top* of the y-axis - Have x & y axes meet at common origin

    //tick marks:
    //This is either or a height or width depending on whether an x-axis or a y-axis tick: 
    var tickLength = 8; 
//Offsets with respect to relevent axis:
    var xTickYOffset = (-1 * tickLength) / 2;
    var yTickXOffset = (-1 * tickLength) / 2;
//tick mark value label offsets:
    var xValYOffset = 2 * tickLength;
    var yValXOffset = (-4 * tickLength);

//
//Other general values:
//Default color of plot elements:
  var lineColor = "#000000"; //black
  var wColor = "#222222";
//Default one thickness of plot elements:
  var lineThick = 1;

//
    //
    //
    //
    //
    // *******************************************
    //
    //
    //  This section has global physics related quantities needed for content of plots
    //
    // Line center, lambda_0 

    //
    //Initialize *physical* quantities needed for various plots - plots are now all in if(){} blocks
    // so all this now has to be initialized ahead of time:
    // Will need this in some if blocks below:
    var iLamMinMax = minMax(logBlam);
    var iLamMax = iLamMinMax[1];
    var iLamMin = iLamMinMax[0];
    var norm = 1.0e15; // y-axis normalization
    var wien = 2.8977721E-1; // Wien's displacement law constant in cm K
    var lamMax = 1.0e7 * (wien / teff);
    lamMax = lamMax.toPrecision(5);
    var lamMaxStr = lamMax.toString(10);

    //
    //Leave uncommented:
    var bNormVega, vNormVega, rNormVega;
    ////Uncomment this block and set input stellar parameters to Vega to re-calibrate 
    //// colors to make Vega's disk centre intensity white 
    //console.log("bNormVega " + bNormVega + " vNormVega  " + vNormVega + " rNormVega " + rNormVega);
   
    //console.log("rrggbbI " + rrggbbI + " minmaxI " + minmaxI + " maxI " + maxI + " renormI " + renormI);
    var saveRGB = []; //intialize
    var saveRadius = 0.0; //initialize
    var radiusScale = 30; //solar_radii-to-pixels!
    var logScale = 50; //amplification factor for log pixels
    // 
    // Star radius in pixels:
    //    var radiusPx = (radiusScale * radius);  //linear radius
    var radiusPx = logScale * logTen(radiusScale * radius); //logarithmic radius
    radiusPx = Math.ceil(radiusPx);
    // LTE Eddington-Barbier limb darkening: I(Tau=0, cos(theta)=t) = B(T(Tau=t))
    var radiusPxI = Math.ceil(radiusPx);
    var radiusStr = numToPxStrng(radiusPxI);
    saveRadius = radiusPxI; // For HRD, plot nine
   
    var numBands = bandFlux.length;
    ////Uncomment this block and set input stellar parameters to Vega to re-calibrate
    //// colors to make Vega's disk centre intensity white
    var bvrVega0 = bandFlux[2] + bandFlux[3] + bandFlux[4];
    var bNormVega = bvrVega0 / bandFlux[2];
    var vNormVega = bvrVega0 / bandFlux[3];
    var rNormVega = bvrVega0 / bandFlux[4];
    //console.log("bNormVega " + bNormVega + " vNormVega  " + vNormVega + " rNormVega " + rNormVega);
//Set Vega disk centre intensity calibration factors (reciprocals of these should total to 1.0):
    bNormVega = 2.223444;
    vNormVega = 3.813167;
    rNormVega = 3.472246;

// Total B + V + R band intensity of prgram object at disk centre:
    var bvr0 = bandFlux[2] + bandFlux[3] + bandFlux[4];
    //console.log("bvr0 " + bvr0);
//Find greatest disk-centre band-integrated I value among B, V, and R for final renormalization:
    var rrI = bandFlux[4] / bvr0 * rNormVega;
    var ggI = bandFlux[3] / bvr0 * vNormVega;
    var bbI = bandFlux[2] / bvr0 * bNormVega;
    //console.log("rrI " + rrI + " ggI " + ggI + " bbI " + bbI);
    var rrggbbI = [rrI, ggI, bbI];
    var minmaxI = minMax(rrggbbI);
    var maxI = minmaxI[1];
    var renormI = rrggbbI[maxI];

    //console.log("i = 3:");
    //console.log("Before renorm: rrI " + rrI + " ggI " + ggI + " bbI " + bbI);
//Renormalize:
    rrI = rrI /renormI;
    ggI = ggI /renormI;
    bbI = bbI /renormI;
    //console.log("After renorm: rrI " + rrI + " ggI " + ggI + " bbI " + bbI);

var RGBArr = [];
    RGBArr.length = 3;
    RGBArr[0] = Math.ceil(255.0 * rrI);
    RGBArr[1] = Math.ceil(255.0 * ggI);
    RGBArr[2] = Math.ceil(255.0 * bbI);
    saveRGB = RGBArr; // For HRD, plot nine

//Variables to comparison to standard solar model from Table 3A of 
// ApJ, 387, 372 (1992) Guenther, D.B, Demarque, P., Kim, Y-C., Pinsonneault, M.
  
    var RGBHexSun = colHex(255.0, 0.0 ,0.0);
    var rSunRef = rRef();
    var numRef = rSunRef.length;
    for (var i=0; i<numRef; i++){
        rSunRef[i] = 1.0e-8 * rSunRef[i] * rSun;
    }

        var opac = 1.0; //opacity
        // RGB color
        var r255Blck = 0;
        var g255Blck = 0;
        var b255Blck = 0; //black 
        // 
        var r255B = 0;
        var g255B = 0;
        var b255B = 255; //blue 
        // 
        var r255G = 0;
        var g255G = 255;
        var b255G = 100; //green
        // 
        var r255P = 100;
        var g255P = 0;
        var b255P = 200; //purple
        // 
        var r255O = 200;
        var g255O = 100;
        var b255O = 0; //orange
        // 
        var r255R = 255;
        var g255R = 0;
        var b255R = 0; //red


    //   var testVal = -1.26832e7;
//

    var xFinesse = 0.0; //default initialization
    var yFinesse = 0.0; //default initialization


//
//  *****   PLOT SEVEN / PLOT 7
//
//

// Plot seven - image of WHITE LIGHT stellar disk
//

        var plotRow = 0;
        var plotCol = 0;


        washerGrid(panelWidth, panelHeight,
            wDiskColor, plotSevenId, cnvsSevenId);


        txtSVGprint("T<tspan style='baseline-shift:sub;'>eff</tspan> white light color (Logarithmic radius)", titleOffsetX, titleOffsetY, lineColor, cnvsSevenId);
            var yCenterCnvs = panelHeight / 2; 
            var xCenterCnvs = panelWidth / 2;
				//JB

// Adjust position to center star:
// Radius is really the *diameter* of the symbol

      //txtSVGprint("<tspan style='font:0.75em;'>\u03b8 = </tspan>", titleOffsetX, titleOffsetY+20, RGBHex, cnvsSevenId);
        //  Loop over limb darkening sub-disks - largest to smallest
	

            // LTE Eddington-Barbier limb darkening: I(Tau=0, cos(theta)=t) = B(T(Tau=t))
			
            var radiusPxICnvs = Math.ceil(radiusPx);

            rrI = bandFlux[4] / bvr0 * rNormVega;
            ggI = bandFlux[3] / bvr0 * vNormVega;
            bbI = bandFlux[2] / bvr0 * bNormVega;
            //console.log("ii " + ii + " rrI " + rrI + " ggI " + ggI + " bbI " + bbI);
            rrI = Math.ceil(255.0 * rrI / renormI);
            ggI = Math.ceil(255.0 * ggI / renormI);
            bbI = Math.ceil(255.0 * bbI / renormI);
            //console.log("Renormalized: rrI " + rrI + " ggI " + ggI + " bbI " + bbI);

            var RGBHex = colHex(rrI, ggI, bbI);
 				//JB
            //console.log("RGBHex " + RGBHex + " RGBHexNext "  + RGBHexNext);

//	if((radiusPxICnvs==radiusPxICnvsNext)){ radiusPxICnvsNext = radiusPxICnvs - 0.9*i/3;}

//create circle of star's radius 
        var circ = document.createElementNS(xmlW3, 'circle');
        circ.setAttributeNS(null, 'cx', xCenterCnvs);
        circ.setAttributeNS(null, 'cy', yCenterCnvs);
        circ.setAttributeNS(null, 'r', radiusPxICnvs);
        //circ.setAttributeNS(null, 'fill', 'url(#grdId)');
        circ.setAttributeNS(null, 'fill', RGBHex);
        //circ.setAttributeNS(xmlns, xmlnsLink, xmlnsLink2);

        //cnvsSevenId.appendChild(grd);
        cnvsSevenId.appendChild(circ);

//

//
//  *****   PLOT TEN / PLOT 10 
//
//

// Plot eight - image of temperature-colored stellar interior cross-section 
//

        var plotRow = 0;
        var plotCol = 1;

        //wDiskColor = "#B0B0B0"; 
        washerGrid(panelWidth, panelHeight,
            wDiskColor, plotTenId, cnvsTenId);

                                //JB
      txtSVGprint("Temperature cross-section", titleOffsetX, titleOffsetY, lineColor, cnvsTenId);
      txtSVGprint("<tspan style='font:0.75em; color:black;'>Black circle: Nuclear burning core boundary </tspan>", titleOffsetX, titleOffsetY+10, lineColor, cnvsTenId);
      txtSVGprint("<tspan style='font:0.75em; color:black;'>Color: log T</tspan>", titleOffsetX, titleOffsetY+20, lineColor, cnvsTenId);

            var yCenterCnvs = panelHeight / 2;
            var xCenterCnvs = panelWidth / 2;
                                //JB

        var limbRadius = Math.ceil(radiusPx);
        var kk = 0; // T legend label
        var tempLbl = " ";

                                //JB


//  Loop over radial shells - largest to smallest

   for (var i = iUpper-1; i >= 1; i--) {

    // shell radius in pixels:
	var rvarSun = rvar[i] / rSun;
        var radiusPx = radiusScale * rvarSun; 
        var radiusPxICnvs = Math.ceil(radiusPx);
	//rvarSun = rvar[i-1] / rSun;
        //var radiusPxNext = radiusScale * rvarSun; 
        //var radiusPxICnvsNext = Math.ceil(radiusPxNext);

        var RGBHex = logTempToRGB(logTemp[i]);
        //var RGBHex = tempToRGB(temp[i]);
        //var RGBHexNext = logTempToRGB(logTemp[i-1]); 

	   //JB
        //console.log("i " + i + " rvarSun " + rvarSun + " radiusPxICnvs " + radiusPxICnvs + " RGBHex " + RGBHex);

//create gradient for each circle
//      if((radiusPxICnvs==radiusPxICnvsNext)){ radiusPxICnvsNext = radiusPxICnvs - 0.9*i/3;}
//      var thisCircR = radiusPxICnvsNext/radiusPxICnvs;

//create circle for each radial shell 
        var circ = document.createElementNS(xmlW3, 'circle');
        circ.setAttributeNS(null, 'cx', xCenterCnvs);
        circ.setAttributeNS(null, 'cy', yCenterCnvs);
        circ.setAttributeNS(null, 'r', radiusPxICnvs);
        //circ.setAttributeNS(null, 'fill', 'url(#grdId)');
        circ.setAttributeNS(null, 'fill', RGBHex);
        //circ.setAttributeNS(xmlns, xmlnsLink, xmlnsLink2);

        //cnvsTenId.appendChild(grd);
        cnvsTenId.appendChild(circ);

                                //JB
            //
        //Temperature color code legend:

        if ((i % 100) == 0.0) {
            tempLbl = (log10E*logTemp[i]).toPrecision(3);
           // txtPrint("<span style='font-size:x-small; color:starRGBHex'>T<sub>Kin</sub> = "
            //        + tempLbl + " K</span>",
            //        titleXPos + 40, 50 + titleYPos + 10 * kk, rrI, ggI, bbI, plotTenId);
            txtSVGprint("<tspan style='font:0.5em; fill:RGBHex;'> log T=" + tempLbl + " K </tspan>", 
		     titleOffsetX, titleOffsetY+30+(15*kk), RGBHex, cnvsTenId);
            kk++;
        } //kk loop - T legend

	   
    // Add circular band for nuclear burning core:

        if ( (i <= iCore+5) && (i >= iCore-5) ){
	    //console.log("i " + i + " rvar[i] " + rvar[i] + " radiusPxICnvs " + radiusPxICnvs);
            var circ = document.createElementNS(xmlW3, 'circle');
            circ.setAttributeNS(null, 'cx', xCenterCnvs);
            circ.setAttributeNS(null, 'cy', yCenterCnvs);
            circ.setAttributeNS(null, 'r', radiusPxICnvs);
            //circ.setAttributeNS(null, 'fill', 'url(#grdId)');
            circ.setAttributeNS(null, 'fill', "#000000");
        //circ.setAttributeNS(xmlns, xmlnsLink, xmlnsLink2);

        //cnvsTenId.appendChild(grd);
            cnvsTenId.appendChild(circ);
        }

    }  // numDeps loop, i

// Add circular band for nuclear burning core:

    

				
    //
    //
    //  *****   PLOT NINE / PLOT 9
    //
    //
    // Plot Nine: HRDiagram


        var plotRow = 1;
        var plotCol = 2;

//background color needs to be finessed so that white-ish stars will stand out:
       if (teff > 6000.0){
  //hotter white or blue-white star - darken the background (default background in #F0F0F0
           wDiskColor =  wDefaultColor;  
       } else {
           wDiskColor = wDefaultColor;
       }
        // WARNING: Teff axis is backwards!!
        var minXData = logTen(100000.0); //K
        var maxXData = logTen(1000.0); //K


        //var xAxisName = "<span title='Logarithmic surface temperature of spherical blackbody radiation emitter of equivalent bolometric surface flux, in Kelvins (K)'> \n\
     //<a href='http://en.wikipedia.org/wiki/Effective_temperature' target='_blank'>\n\
     //Log<sub>10</sub> <em>T</em><sub>eff</sub></a> \n\
     //(<a href='http://en.wikipedia.org/wiki/Kelvin' target='_blank'>K</a>)</span>";
        var xAxisName = "Log<tspan style='baseline-shift:sub;'>10</tspan> <tspan style='font:italic;'>T</tspan><tspan style='baseline-shift:sub;'>eff</tspan> (K)";
        //var numYTicks = 6;
        var minYData = -6.0; //solar luminosities;
        var maxYData = 7.0; //solar luminosities


    //    var yAxisName = "<span title='Logarithmic Bolometric luminosity'>\n\
    // <a href='http://en.wikipedia.org/wiki/Luminosity' target='_blank'>\n\
    // Log<sub>10</sub><em>L</em><sub>Bol</sub></a></span><br />  \n\
    // <span title='Solar luminosities'>\n\
    // <a href='http://en.wikipedia.org/wiki/Solar_luminosity' target='_blank'>\n\
    // <em>L</em><sub>Sun</sub></a></span> ";
            var yAxisName = "Log<tspan style='baseline-shift:sub;'>10</tspan> Luminosity (L<tspan style='baseline-shift:sub;'>Sun</tspan>)";
        //
        var fineness = "fine";
				//JB
       //var panelOrigin = washer(plotRow, plotCol, wDiskColor, plotNineId, cnvsNineId);

				//JB
        //panelX = panelOrigin[0];
        //panelY = panelOrigin[1];

        washerGrid(panelWidth, panelHeight,
            wDiskColor, plotNineId, cnvsNineId);
				//JB
        var xAxisParams = XAxis(panelWidth, panelHeight, xAxisLength, yAxisLength,
                minXData, maxXData, xAxisName, fineness,
                plotNineId, cnvsNineId);

        var yAxisParams = YAxis(panelWidth, panelHeight, xAxisLength, yAxisLength,
                minYData, maxYData, yAxisName, fineness,
                plotNineId, cnvsNineId);
				//JB
        //
//        xOffset = xAxisParams[0];
//        yOffset = yAxisParams[4];
        var rangeXData9 = xAxisParams[1];
        var deltaXData9 = xAxisParams[2];
        var deltaXPxl9 = xAxisParams[3];
        var rangeYData9 = yAxisParams[1];
        var deltaYData9 = yAxisParams[2];
        var deltaYPxl9 = yAxisParams[3];
//        var xLowerYOffset = xAxisParams[5];
        var minXData9 = xAxisParams[6]; //updated value
        var minYData9 = yAxisParams[6]; //updated value
        var maxXData9 = xAxisParams[7]; //updated value
        var maxYData9 = yAxisParams[7]; //updated value     
        //
        //titleX = panelX + titleOffsetX;
        //titleY = panelY + titleOffsetY;
				//JB
        //txtPrint("<span style='font-size:normal; color:blue'><a href='http://www.ap.smu.ca/~ishort/hrdtest3.html' target='_blank'>H-R Diagram</a></span>",
        //        titleOffsetX, titleOffsetY, 300, lineColor, plotNineId);
        txtSVGprint("Hertzsprung-Russell (H-R) Diagram", titleOffsetX, titleOffsetY, lineColor, cnvsNineId);	
			//JB
        // *********  Input stellar data

        //Sun

        var sunClass = "G2";
        var sunTeff = 5778;
        // var sunTeff = 10000; //debug test
        var sunB_V = 0.656;
        var sunM_v = 4.83;
        var sunM_Bol = 4.75;
        var sunRad = 1.0;
        // var sunRad = 10.0; //debug test
        var logSunLum = 2.5 * logTen(1.0); //log Suns luminosity in solar luminosities 


        // Carroll & Ostlie, 2nd Ed. , Appendix G:

        //Main sequence

        var msClass = ["O5", "O6", "O7", "O8", "B0", "B1", "B2", "B3", "B5", "B6", "B7", "B8", "B9", "A0", "A1", "A2", "A5", "A8", "F0", "F2", "F5", "F8", "G0", "G2", "G8", "K0", "K1", "K3", "K4", "K5", "K7", "M0", "M1", "M2", "M3", "M4", "M5", "M6", "M7"];
        var msTeffs = [42000, 39500, 37500, 35800, 30000, 25400, 20900, 18800, 15200, 13700, 12500, 11400, 10500, 9800, 9400, 9020, 8190, 7600, 7300, 7050, 6650, 6250, 5940, 5790, 5310, 5150, 4990, 4690, 4540, 4410, 4150, 3840, 3660, 3520, 3400, 3290, 3170, 3030, 2860];
        var msB_V = [-0.33, -0.33, -0.32, -0.32, -0.30, -0.26, -0.24, -0.20, -0.17, -0.15, -0.13, -0.11, -0.07, -0.02, +0.01, +0.05, +0.15, +0.25, +0.30, +0.35, +0.44, +0.52, +0.58, +0.63, +0.74, +0.81, +0.86, +0.96, +1.05, +1.15, +1.33, +1.40, +1.46, +1.49, +1.51, +1.54, +1.64, +1.73, +1.80];
        var msM_v = [-5.1, -5.1, -4.9, -4.6, -3.4, -2.6, -1.6, -1.3, -0.5, -0.1, +0.3, +0.6, +0.8, +1.1, +1.3, +1.5, +2.2, +2.7, +3.0, +3.4, +3.9, +4.3, +4.7, +4.9, +5.6, +5.7, +6.0, +6.5, +6.7, +7.1, +7.8, +8.9, +9.6, 10.4, 11.1, 11.9, 12.8, 13.8, 14.7];
        var msBC = [-4.40, -3.93, -3.68, -3.54, -3.16, -2.70, -2.35, -1.94, -1.46, -1.21, -1.02, -0.80, -0.51, -0.30, -0.23, -0.20, -0.15, -0.10, -0.09, -0.11, -0.14, -0.16, -0.18, -0.20, -0.40, -0.31, -0.37, -0.50, -0.55, -0.72, -1.01, -1.38, -1.62, -1.89, -2.15, -2.38, -2.73, -3.21, -3.46];
        var msMass = [60, 37, 29, 23, 17.5, 13.0, 10.0, 7.6, 5.9, 4.8, 4.3, 3.8, 3.2, 2.9, 2.5, 2.2, 2.0, 1.7, 1.6, 1.5, 1.4, 1.2, 1.05, 0.90, 0.83, 0.79, 0.75, 0.72, 0.69, 0.67, 0.55, 0.51, 0.43, 0.40, 0.33, 0.26, 0.21, 0.13, 0.10];
        //var msRads = [13.4 ,12.2 ,11.0 ,10.0 , 6.7 , 5.2 , 4.1 , 3.8 , 3.2 , 2.9 , 2.7 , 2.5 , 2.3 , 2.2 , 2.1 , 2.0 , 1.8 , 1.5 , 1.4 , 1.3 , 1.2 , 1.1 , 1.06, 1.03, 0.96, 0.93, 0.91, 0.86, 0.83, 0.80, 0.74, 0.63, 0.56, 0.48, 0.41, 0.35, 0.29, 0.24, 0.20];
        //var msM_Bol = [-9.51,-9.04,-8.60,-8.18,-6.54,-5.26,-3.92,-3.26,-1.96,-1.35,-0.77,-0.22,+0.28,+0.75,+1.04,+1.31,+2.02,+2.61,+2.95,+3.27,+3.72,+4.18,+4.50,+4.66,+5.20,+5.39,+5.58,+5.98,+6.19,+6.40,+6.84,+7.52,+7.99,+8.47,+8.97,+9.49,10.1 ,10.6 ,11.3];

        // Main sequence data processing:

        var msNum = msClass.length;
        var msM_Bol = [];
        var logR45 = [];
        var logR = [];
        var msRads = [];
        var msLogLum = [];
        msM_Bol.length = msNum;
        logR45.length = msNum;
        logR.length = msNum;
        msRads.length = msNum;
        msLogLum.length = msNum;
        // Calculate radii in solar radii:
        // For MS stars, do the Luminosity as well

        for (var i = 0; i < msNum; i++) {

            msM_Bol[i] = msM_v[i] + msBC[i];
            var msTeffSol = msTeffs[i] / sunTeff;
            logR45[i] = 2.5 * logSunLum + sunM_Bol - 10.0 * logTen(msTeffSol) - msM_Bol[i];
            logR[i] = logR45[i] / 4.5;
            msRads[i] = Math.exp(Math.LN10 * logR[i]); //No base ten exponentiation in JS!
            var msLogL = (sunM_Bol - msM_Bol[i]) / 2.5;
            // Round log(Lum) to 1 decimal place:
            msLogL = 10.0 * msLogL;
            msLogL = Math.floor(msLogL);
            msLogLum[i] = msLogL / 10.0;
        } // end i loop


// Giants:

        var rgbClass = ["O5", "O6", "O7", "O8", "B0", "B1", "B2", "B3", "B5", "B6", "B7", "B8", "B9", "A0", "A1", "A2", "A5", "A8", "F0", "F2", "F5", "G0", "G2", "G8", "K0", "K1", "K3", "K4", "K5", "K7", "M0", "M1", "M2", "M3", "M4", "M5", "M6"];
        var rgbTeffs = [39400, 37800, 36500, 35000, 29200, 24500, 20200, 18300, 15100, 13800, 12700, 11700, 10900, 10200, 9820, 9460, 8550, 7830, 7400, 7000, 6410, 5470, 5300, 4800, 4660, 4510, 4260, 4150, 4050, 3870, 3690, 3600, 3540, 3480, 3440, 3380, 3330];
        var rgbB_V = [-0.32, -0.32, -0.32, -0.31, -0.29, -0.26, -0.24, -0.20, -0.17, -0.15, -0.13, -0.11, -0.07, -0.03, +0.01, +0.05, +0.15, +0.25, +0.30, +0.35, +0.43, +0.65, +0.77, +0.94, +1.00, +1.07, +1.27, +1.38, +1.50, +1.53, +1.56, +1.58, +1.60, +1.61, +1.62, +1.63, +1.52];
        var rgbM_v = [-5.9, -5.7, -5.6, -5.5, -4.7, -4.1, -3.4, -3.2, -2.3, -1.8, -1.4, -1.0, -0.6, -0.4, -0.2, -0.1, +0.6, +1.0, +1.3, +1.4, +1.5, +1.3, +1.3, +1.0, +1.0, +0.9, +0.8, +0.8, +0.7, +0.4, +0.0, -0.2, -0.4, -0.4, -0.4, -0.4, -0.4];
        var rgbBC = [-4.05, -3.80, -3.58, -3.39, -2.88, -2.43, -2.02, -1.60, -1.30, -1.13, -0.97, -0.82, -0.71, -0.42, -0.29, -0.20, -0.14, -0.10, -0.11, -0.11, -0.14, -0.20, -0.27, -0.42, -0.50, -0.55, -0.76, -0.94, -1.02, -1.17, -1.25, -1.44, -1.62, -1.87, -2.22, -2.48, -2.73];
        //var rgbRads = [18.5,16.8,15.4,14.3,11.4,10.0, 8.6, 8.0, 6.7, 6.1, 5.5, 5.0, 4.5, 4.1, 3.9, 3.7, 3.3, 3.1, 3.2, 3.3, 3.8, 6.0, 6.7, 9.6,10.9,12.5,16.4,18.7,21.4,27.6,39.3,48.6,58.5,69.7,82.0,96.7,16];
        //var rgbM_Bol = [-9.94,-9.55,-9.20,-8.87,-7.58,-6.53,-5.38,-4.78,-3.56,-2.96,-2.38,-1.83,-1.31,-0.83,-0.53,-0.26,+0.44,+0.95,+1.17,+1.31,+1.37,+1.10,+1.00,+0.63,+0.48,+0.32,-0.01,-0.18,-0.36,-0.73,-1.28,-1.64,-1.97,-2.28,-2.57,-2.86,-3.18];

        // RGB sequence data processing:

        var rgbNum = rgbClass.length;
        var rgbM_Bol = [];
        var logR45 = [];
        var logR = [];
        var rgbRads = [];
        var rgbLogLum = [];
        rgbM_Bol.length = rgbNum;
        logR45.length = rgbNum;
        logR.length = rgbNum;
        rgbRads.length = rgbNum;
        // Calculate radii in solar radii:

        for (var i = 0; i < rgbNum; i++) {

            rgbM_Bol[i] = rgbM_v[i] + rgbBC[i];
            var rgbTeffSol = rgbTeffs[i] / sunTeff;
            logR45[i] = 2.5 * logSunLum + sunM_Bol - 10.0 * logTen(rgbTeffSol) - rgbM_Bol[i];
            logR[i] = logR45[i] / 4.5;
            rgbRads[i] = Math.exp(Math.LN10 * logR[i]); //No base ten exponentiation in JS!

            var rgbLogL = (sunM_Bol - rgbM_Bol[i]) / 2.5;
            // Round log(Lum) to 1 decimal place:
            rgbLogL = 10.0 * rgbLogL;
            rgbLogL = Math.floor(rgbLogL);
            rgbLogLum[i] = rgbLogL / 10.0;
        } // end i loop


// No! Too bright for what GrayStar can model!
// //Supergiants:
//
 var sgbClass = ["O5", "O6", "O7", "O8", "B0", "B1", "B2", "B3", "B5", "B6", "B7", "B8", "B9", "A0", "A1", "A2", "A5", "A8", "F0", "F2", "F5", "F8", "G0", "G2", "G8", "K0", "K1", "K3", "K4", "K5", "K7", "M0", "M1", "M2", "M3", "M4", "M5", "M6"];
 var sgbTeffs = [40900, 38500, 36200, 34000, 26200, 21400, 17600, 16000, 13600, 12600, 11800, 11100, 10500, 9980, 9660, 9380, 8610, 7910, 7460, 7030, 6370, 5750, 5370, 5190, 4700, 4550, 4430, 4190, 4090, 3990, 3830, 3620, 3490, 3370, 3210, 3060, 2880, 2710];
 var sgbB_V = [-0.31, -0.31, -0.31, -0.29, -0.23, -0.19, -0.17, -0.13, -0.10, -0.08, -0.05, -0.03, -0.02, -0.01, +0.02, +0.03, +0.09, +0.14, +0.17, +0.23, +0.32, +0.56, +0.76, +0.87, +1.15, +1.24, +1.30, +1.46, +1.53, +1.60, +1.63, +1.67, +1.69, +1.71, +1.69, +1.76, +1.80, +1.86];
  var sgbM_v = [-6.5, -6.5, -6.6, -6.6, -6.9, -6.9, -6.7, -6.7, -6.6, -6.4, -6.3, -6.3, -6.3, -6.3, -6.3, -6.3, -6.3, -6.4, -6.4, -6.4, -6.4, -6.4, -6.3, -6.3, -6.1, -6.1, -6.0, -5.9, -5.8, -5.7, -5.6, -5.8, -5.8, -5.8, -5.5, -5.2, -4.8, -4.9];
 var sgbBC = [-3.87, -3.74, -3.48, -3.35, -2.49, -1.87, -1.58, -1.26, -0.95, -0.88, -0.78, -0.66, -0.52, -0.41, -0.32, -0.28, -0.13, -0.03, -0.01, 0.00, -0.03, -0.09, -0.15, -0.21, -0.42, -0.50, -0.56, -0.75, -0.90, -1.01, -1.20, -1.29, -1.38, -1.62, -2.13, -2.75, -3.47, -3.90];
//  
//  //var sgbRads = [21,  22,  23,  25,  31,  37,  42,  45,  51,  53,  56,  58,  61,  64,  67,  69,  78,  91, 102, 114, 140, 174, 202, 218, 272, 293, 314, 362, 386, 415, 473, 579, 672, 791, 967,1220,1640,2340];
//  //var sgbM_Bol = [-10.4,-10.2,-10.1, -9.9, -9.3, -8.8, -8.2, -7.9, -7.5, -7.3, -7.1, -6.9, -6.8, -6.7, -6.6, -6.5, -6.4, -6.4, -6.4, -6.4, -6.4, -6.4, -6.4, -6.4, -6.5, -6.5, -6.5, -6.6, -6.7, -6.7, -6.8, -7.0, -7.2, -7.4, -7.6, -7.9, -8.3, -8.8];
// 
  // SGB sequence data processing:
 
  var sgbNum = sgbClass.length;
 
  var sgbM_Bol = [];
  var logR45 = [];
  var logR = [];
  var sgbRads = [];
  var sgbLogLum = [];
  
 sgbM_Bol.length = sgbNum;
  logR45.length = sgbNum;
  logR.length = sgbNum;
   sgbRads.length = sgbNum;
  
   
  // Calculate radii in solar radii:
   
  for (var i = 0; i < sgbNum; i++) {
  
 sgbM_Bol[i] = sgbM_v[i] + sgbBC[i];
  var sgbTeffSol = sgbTeffs[i] / sunTeff;
  
  logR45[i] = 2.5 * logSunLum + sunM_Bol - 10.0 * logTen(sgbTeffSol) - sgbM_Bol[i];
  logR[i] = logR45[i] / 4.5;
  sgbRads[i] = Math.exp(Math.LN10 * logR[i]);  //No base ten exponentiation in JS!
  
  var sgbLogL = (sunM_Bol - sgbM_Bol[i]) / 2.5;
  // Round log(Lum) to 1 decimal place:
  sgbLogL = 10.0 * sgbLogL;
  sgbLogL = Math.floor(sgbLogL);
  sgbLogLum[i] = sgbLogL / 10.0;
  
  } // end i loop

//Lines of constant radius first
//
     var RGBHex = colHex(100, 100, 100);
     var dSizeCnvs = 2;
     var deltaLog10Teff = 0.2;  //Delta Log(Teff) in K
     var numRadLine = (maxXData9 - minXData9) / deltaLog10Teff;
     numRadLine = Math.abs(Math.ceil(numRadLine));
     //console.log(numRadLine);
     var log10TeffSun = logTen(teffSun);
     var thisLog10Teff, thisLog10TeffSol, log10L;

     var thisLog10Rad;
     var HRradii = [0.01, 0.1, 1.0, 10.0, 100.0, 1000.0]; //solar radii
     var numRad = HRradii.length;
     //console.log("numRad " + numRad);

     for (var r = 0; r < numRad; r++){
 
       thisLog10Rad = logTen(HRradii[r]); //solar units
//Seed first data point
        thisLog10Teff = minXData9; //K
        thisLog10TeffSol = thisLog10Teff - log10TeffSun; //solar units  
        var xTickPosCnvs = xAxisLength * (thisLog10Teff - minXData9) / rangeXData9; // pixels
        var lastXShiftCnvs = xAxisXCnvs + xTickPosCnvs;
        log10L = (4.0*thisLog10TeffSol) + (2.0*thisLog10Rad);
        var yTickPosCnvs = yAxisLength * (log10L - minYData9) / rangeYData9;
        var lastYShiftCnvs = (yAxisYCnvs + yAxisLength) - yTickPosCnvs;

    for (var i = 1; i < numRadLine; i++){
   //Caution: Teff axis backwards so minXData > maxXData:
        thisLog10Teff = minXData9 - (i * deltaLog10Teff); //K
        thisLog10TeffSol = thisLog10Teff - log10TeffSun; //solar units  
        log10L = (4.0*thisLog10TeffSol) + (2.0*thisLog10Rad);

        //console.log("thisLog10Teff " + thisLog10Teff + " log10L " + log10L);
 
          var xTickPosCnvs = xAxisLength * (thisLog10Teff - minXData9) / rangeXData9; // pixels   

          // horizontal position in pixels - data values increase rightward:
          var xShiftCnvs = xAxisXCnvs + xTickPosCnvs;

          var yTickPosCnvs = yAxisLength * (log10L - minYData9) / rangeYData9;
          // vertical position in pixels - data values increase upward:
          var yShiftCnvs = (yAxisYCnvs + yAxisLength) - yTickPosCnvs;

           var thisLine = document.createElementNS(xmlW3, 'line');
           thisLine.setAttributeNS(null, 'x1', lastXShiftCnvs);
           thisLine.setAttributeNS(null, 'x2', xShiftCnvs); 
           thisLine.setAttributeNS(null, 'y1', lastYShiftCnvs);
           thisLine.setAttributeNS(null, 'y2', yShiftCnvs); 
           thisLine.setAttributeNS(null, 'stroke', 'black');
           thisLine.setAttributeNS(null, 'stroke-width', 1);

           cnvsNineId.appendChild(thisLine);

           
            lastXShiftCnvs = xShiftCnvs;
            lastYShiftCnvs = yShiftCnvs;
  } //i loop over Teff
				//JB
          //txtPrint("<span style='font-size:xx-small'>" + HRradii[r] + " R<sub>Sun</sub></span>",
          //          xShiftCnvs+5, yShiftCnvs-5, 300, RGBHex, plotNineId);
          txtSVGprint(" " + HRradii[r], xShiftCnvs+5, yShiftCnvs-5, RGBHex, cnvsNineId);
				//JB
 } //r loop over radii
 

//Data loops - plot the result!

//MS stars

        var dSizeCnvs = 2.0; //plot point size
        var opac = 0.7; //opacity
        // RGB color
        var r255 = 0;
        var g255 = 0;
        var b255 = 0; 
        var RGBHex = colHex(r255, r255, r255);

        //Populate vectors for general plot routine:
        var RGBHexV = [];
        RGBHexV.length = msNum;
        var logMsTeffs = [];
        logMsTeffs.length = msNum;
        for (var i = 0; i < msNum; i++){
           RGBHexV[i] = RGBHex;
           logMsTeffs[i] = logTen(msTeffs[i]);
        }

     plot(cnvsNineId, logMsTeffs, msLogLum,
          panelWidth, panelHeight, xAxisLength, yAxisLength,
          minXData9, maxXData9, rangeXData9, minYData9, maxYData9, rangeYData9,
          RGBHexV, 'o', dSizeCnvs);

//RGB stars

// RGB color
        var RGBHex = colHex(r255, r255, r255);
        //Populate vectors for general plot routine:
        var RGBHexV = [];
        RGBHexV.length = rgbNum;
        var logRgbTeffs = [];
        logRgbTeffs.length = rgbNum;
        for (var i = 0; i < rgbNum; i++){
           RGBHexV[i] = RGBHex;
           logRgbTeffs[i] = logTen(rgbTeffs[i]);
        }

     plot(cnvsNineId, logRgbTeffs, rgbLogLum,
          panelWidth, panelHeight, xAxisLength, yAxisLength,
          minXData9, maxXData9, rangeXData9, minYData9, maxYData9, rangeYData9,
          RGBHexV, 'o', dSizeCnvs);


// //SGB stars
// 
        var RGBHex = colHex(r255, r255, r255);
        //Populate vectors for general plot routine:
        var RGBHexV = [];
        RGBHexV.length = sgbNum;
        var logSgbTeffs = [];
        logSgbTeffs.length = sgbNum;
        for (var i = 0; i < sgbNum; i++){
           RGBHexV[i] = RGBHex;
           logSgbTeffs[i] = logTen(sgbTeffs[i]);
        }

     plot(cnvsNineId, logSgbTeffs, sgbLogLum,
          panelWidth, panelHeight, xAxisLength, yAxisLength,
          minXData9, maxXData9, rangeXData9, minYData9, maxYData9, rangeYData9,
          RGBHexV, 'o', dSizeCnvs);

    //cnvsNineId.addEventListener("mouseover", function() { 
    cnvsNineId.addEventListener("click", function() {
       //dataCoords(event, plotNineId);
       var xyString = dataCoords(event, cnvsNineId, xAxisLength, minXData9, rangeXData9, xAxisXCnvs,
                               yAxisLength, minYData9, rangeYData9, yAxisYCnvs);
       //txtPrint(xyString, titleOffsetX+200, titleOffsetY+320, 150, lineColor, plotNineId);
       txtSVGprint(xyString, titleOffsetX+200, titleOffsetY+320, lineColor, cnvsNineId);
    });



// Now overplot our star:
        var xTickPosCnvs = xAxisLength * (logTen(teff) - minXData9) / rangeXData9; // pixels   
        // horizontal position in pixels - data values increase rightward:
        var xShiftCnvs = xAxisXCnvs + xTickPosCnvs;
//
        var yTickPosCnvs = yAxisLength * (logTen(bolLum) - minYData9) / rangeYData9;
        // vertical position in pixels - data values increase upward:
        var yShiftCnvs = (yAxisYCnvs + yAxisLength) - yTickPosCnvs;
        //Take color and radius from the last step of the star rendering loop (plot Seve) - that should be the inner-most disk:
        var radiusPxThis = saveRadius / 5;
        if (radiusPxThis < 1){
            radiusPxThis = 1;
            }
    
        var rrI = saveRGB[0];
        var ggI = saveRGB[1];
        var bbI = saveRGB[2];
        var RGBHex = colHex(rrI, ggI, bbI);

//
			//JB
            var x4 = logTen(teff);
            var y4 = logTen(bolLum);
//create a circle representing our star
	    var dot = document.createElementNS(xmlW3, 'circle');
	    dot.setAttributeNS(null, 'cx', xShiftCnvs);
            dot.setAttributeNS(null, 'cy', yShiftCnvs);
            dot.setAttributeNS(null, 'r', 1.1 * radiusPxThis);
            dot.setAttributeNS(null, 'stroke', "white");
            dot.setAttributeNS(null, 'fill', wDefaultColor);
            dot.setAttributeNS(null, 'opacity', 0.5);
            cnvsNineId.appendChild(dot);

	    //dot4.setAttributeNS(xmlns,xmlnsLink,xmlnsLink2);

//create another circle behind our star to make it more visable

	    var dot = document.createElementNS(xmlW3, 'circle');
            dot.setAttributeNS(null, 'cx', xShiftCnvs);
            dot.setAttributeNS(null, 'cy', yShiftCnvs);
            dot.setAttributeNS(null, 'r', 1.05 * radiusPxThis);
	    dot.setAttributeNS(null, 'stroke', RGBHex);
            dot.setAttributeNS(null, 'fill', RGBHex);
            dot.setAttributeNS(null, 'opacity', 0.5);
//event listeners added directly onto the one circle
	    //dot5.setAttributeNS(xmlns,xmlnsLink,xmlnsLink2);
            cnvsNineId.appendChild(dot);
	    

			//JB

        //Now overplot Luminosity class markers:

            //I
        var xShift = xAxisXCnvs + xAxisLength * (logTen(sgbTeffs[sgbNum-1]) - minXData9) / rangeXData9; // pixels 
        var yShift = (yAxisYCnvs + yAxisLength) - (yAxisLength * (sgbLogLum[sgbNum - 1] - minYData9) / rangeYData9);
				//JB
        //txtPrint("<span style='font-size:normal'><a href='http://en.wikipedia.org/wiki/Stellar_classification' target='_blank'>\n\
//I</a></span>", xShift, yShift, 300, lineColor, plotNineId);
        txtSVGprint("I", xShift, yShift, lineColor, cnvsNineId);
				//JB
        //III
        xShift = xAxisXCnvs + xAxisLength * (logTen(rgbTeffs[rgbNum-1]) - minXData9) / rangeXData9; // pixels 
        yShift = (yAxisYCnvs + yAxisLength) - (yAxisLength * (rgbLogLum[rgbNum - 8] - minYData9) / rangeYData9);
				//JB
//        txtPrint("<span style='font-size:normal'><a href='http://en.wikipedia.org/wiki/Stellar_classification' title='Giants' target='_blank'>\n\
//     III</a></span>", xShift, yShift, 300, lineColor, plotNineId);
        txtSVGprint("III", xShift, yShift, lineColor, cnvsNineId);
				//JB
        //V
        xShift = xAxisXCnvs + xAxisLength * (logTen(msTeffs[msNum-1]) - minXData9) / rangeXData9; // pixels 
        yShift = (yAxisYCnvs + yAxisLength) - (yAxisLength * (msLogLum[msNum - 8] - minYData9) / rangeYData9);
				//JB
        //txtPrint("<span style='font-size:normal'><a href='http://en.wikipedia.org/wiki/Stellar_classification' title='Main Sequence, Dwarfs' target='_blank'>\n\
     //V</a></span>", xShift, yShift, 300, lineColor, plotNineId);
     txtSVGprint("V", xShift, yShift, lineColor, cnvsNineId);


	
    //
    //
    //  *****   PLOT ONE / PLOT 1
    //
    //
    // Plot one: log(r) vs log(rho)

    if ((ifShowStruc === true)) {

        var plotRow = 3;
        var plotCol = 3;
//Avoid center of star (i=0): log(r=0) undefined:
        //var minXData = log10E * logRvar[1];
        //var maxXData = log10E * logRvar[iUpper - 1];
        var minXData = rvarMm[0];
        var maxXData = rvarMm[iUpper - 1];
        //console.log("minXData " + minXData + " maxXData " + maxXData);


//var xAxisName = "Log<tspan style='baseline-shift:sub;'>10</tspan> <tspan style='font:italic;'>r</tspan><tspan style='baseline-shift:sub;'></tspan> (cm)";
        var xAxisName = "<tspan style='font:italic;'>r</tspan><tspan style='baseline-shift:sub;'></tspan> (Mm)";
        // From Hydrostat.hydrostat:
        //var numYTicks = 6;
        // Build total P from P_Gas & P_Rad:
        //var minYData = log10E * logRho[0] - 2.0; // Avoid upper boundary condition [i]=0
        //var maxYData = log10E * logRho[iUpper - 1];
        var minYData = log10E * logRho[iUpper - 1];
        var maxYData = log10E * logRho[0];
        //var yAxisName = "Log<sub>10</sub> <em>P</em> <br />(dynes <br />cm<sup>-2</sup>)";
        var yAxisName = "Log<tspan style='baseline-shift:sub;'>10</tspan> <tspan style='font:italic;'>\u03c1</tspan> (g cm<tspan style='baseline-shift:super;'>-3</tspan>)";
        //var yAxisName = "Log10 \u03c1 (g cm-3)";
        //console.log("minYData " + minYData + " maxYData " + maxYData);
        //washer(xRange, xOffset, yRange, yOffset, wDefaultColor, plotOneId);

        washerGrid(panelWidth, panelHeight,
            wDiskColor, plotOneId, cnvsOneId);

        var fineness = "normal";
        //var cnvsCtx = washer(plotRow, plotCol, wDefaultColor, plotOneId, cnvsId);
				//JB
        //var panelOrigin = washer(plotRow, plotCol, wDefaultColor, plotOneId, cnvsOneId);
				//JB
        //panelX = panelOrigin[0];
        //panelY = panelOrigin[1];
	//cnvsOneId.setAttribute('fill', wDefaultColor);
//console.log("panelWidth " + panelWidth + " panelHeight " + panelHeight + " xAxisLength " + xAxisLength + " yAxisLength " + yAxisLength);
//console.log("minXData " + minXData + " maxXData " + maxXData + " minYData " + minYData + " maxYData " + maxYData + " fineness " + fineness);
        var xAxisParams = XAxis(panelWidth, panelHeight, xAxisLength, yAxisLength,
                minXData, maxXData, xAxisName, fineness,
                plotOneId, cnvsOneId);
        var yAxisParams = YAxis(panelWidth, panelHeight, xAxisLength, yAxisLength,
                minYData, maxYData, yAxisName,
                fineness, plotOneId, cnvsOneId);
        //xOffset = xAxisParams[0];
        //yOffset = xAxisParams[4];
        var rangeXData1 = xAxisParams[1];
        var deltaXData1 = xAxisParams[2];
        var deltaXPxl1 = xAxisParams[3];
        var rangeYData1 = yAxisParams[1];
        var deltaYData1 = yAxisParams[2];
        var deltaYPxl1 = yAxisParams[3];
        var xLowerYOffset1 = xAxisParams[5];
        var minXData1 = xAxisParams[6]; //updated value
        var minYData1 = yAxisParams[6]; //updated value
        var maxXData1 = xAxisParams[7]; //updated value
        var maxYData1 = yAxisParams[7]; //updated value        
//console.log("rangeXData1 " +  rangeXData1 + " deltaXData1 " + deltaXData1 + " deltaXPxl1 " + deltaXPxl1 + " rangeYData1 " + rangeYData1 + " deltaYData1 " + deltaYData1 + " deltaYPxl1 " + deltaYPxl1 + " xLowerYOffset1 " + xLowerYOffset1 + " minXData1 " + minXData1 + " minYData1 " + minYData1 + " maxXData1 " + maxXData1 + " maxYData1 " + maxYData1);
        yFinesse = 0;       
        xFinesse = 0;       
        //
        //titleX = panelX + titleOffsetX;
        //titleY = panelY + titleOffsetY;
					//JB
        txtSVGprint("log Mass density: <tspan style='fill:blue;><tspan style='font:italic;'>\u03c1</tspan><tspan style='baseline-shift:sub;'></tspan></tspan> ",
                titleOffsetX, titleOffsetY, lineColor, cnvsOneId);

					//JB
        //Data loop - plot the result!

        var dSizeCnvs = 2.0; //plot point size
        var dSizeGCnvs = 1.0;
        var opac = 1.0; //opacity
        // RGB color
        var r255 = 0;
        var g255 = 0;
        var b255 = 0; //black 
        // Rho:
        var r255B = 0;
        var g255B = 0;
        var b255B = 255; //blue 

// Generally, plot every 10th depth:
	var lengthPlotV = Math.round(iUpper/10);
	//console.log("iUpper " + iUpper + " lengthPlotV " + lengthPlotV);
        var RGBHexV = [];
        RGBHexV.length = lengthPlotV;
        var xVec = [];
        xVec.length = lengthPlotV;
        var yVec = [];
        yVec.length = lengthPlotV;

    //Initialize plot argument arrays to valid value:
     for (var i = 0; i < lengthPlotV; i++){
        RGBHexV[i] = 0x000000;
	xVec[i] = rvarMm[0];
	yVec[i] = log10E * logRho[0]; 
     }

//        console.log("iUpper " + iUpper + " logRvar[iUpper-1] " + log10E*logRvar[iUpper-1] + " logRho[iUpper-1] " + log10E*logRho[iUpper-1]);
       j = 0;
       for (var i = 0; i < iUpper; i+=10){
           RGBHexV[j] = colHex(r255B, g255B, b255B);
           //xVec[i] = log10E * logRvar[i];
           xVec[j] = rvarMm[i];
           yVec[j] = log10E * logRho[i];
           //if ( (i%10) == 0 ){
            // console.log("j " + j + " i " + i + " logRvar[i] " + log10E*logRvar[i] + " logRho[i] " + log10E*logRho[i]);
	  // }
	   j++;
        }

     plot(cnvsOneId, xVec, yVec,
          panelWidth, panelHeight, xAxisLength, yAxisLength,
          minXData1, maxXData1, rangeXData1, minYData1, maxYData1, rangeYData1,
          RGBHexV, '-', dSizeCnvs);

//Comparison to standard solar model from Table 3A of 
// ApJ, 387, 372 (1992) Guenther, D.B, Demarque, P., Kim, Y-C., Pinsonneault, M.
    if (ifStandardSun){
       var rhoSunRef = rhoRef();
       var logRhoSunRef = [];
       logRhoSunRef.length = numRef;
       for (var i=0; i<numRef; i++){
          logRhoSunRef[i] = log10E*Math.log(rhoSunRef[i]);
       }
       plot(cnvsOneId, rSunRef, logRhoSunRef,
          panelWidth, panelHeight, xAxisLength, yAxisLength,
          minXData1, maxXData1, rangeXData1, minYData1, maxYData1, rangeYData1,
          RGBHexSun, 'o', dSizeCnvs);
    }

// nuclear burning core radius cross-hair

        var barWidth = 1.0;
        var barColor = "#777777";
        yFinesse = 0.0;
                                //JB
        xShift = YBar(rvarMm[iCore], minXData1, maxXData1, barWidth, yAxisLength,
                panelWidth, panelHeight, xAxisLength, yAxisLength,
                yFinesse, barColor, plotOneId, cnvsOneId);
        barHeight = 1.0;
        yShift = XBar(log10E*logRho[iCore], minYData1, maxYData1, xAxisLength, barHeight,
                panelWidth, panelHeight, xAxisLength, yAxisLength,
                xFinesse, barColor, plotOneId, cnvsOneId);
        //txtPrint("<span style='font-size:small; color:#444444'><em>&#964</em><sub>Ros</sub>=1</span>",
        //        xShift, yShift, 300, lineColor, plotOneId);
        txtSVGprint("<tspan style='font:0.75em; fill:#444444;'><tspan style='font:italic;'>r</tspan><tspan style='baseline-shift:sub;'>Core</tspan></tspan>",
                (xShift+5), (yShift-15), lineColor, cnvsOneId);


    //cnvsOneId.addEventListener("mouseover", function() { 
    cnvsOneId.addEventListener("click", function() {
       //dataCoords(event, plotOneId);
       var xyString = dataCoords(event, cnvsOneId, xAxisLength, minXData1, rangeXData1, xAxisXCnvs,
                               yAxisLength, minYData1, rangeYData1, yAxisYCnvs);
       //txtPrint(xyString, titleOffsetX+200, titleOffsetY+320, 150, lineColor, plotOneId);
       txtSVGprint(xyString, titleOffsetX+200, titleOffsetY+320, lineColor, cnvsOneId);
    }); 


				//JB
    }

					
	
    //
    //
    //  *****   PLOT THREE / PLOT 3
    //
    //
    // Plot three: log(r) vs log(Pressure)

    if ((ifShowStruc === true)) {

        var plotRow = 3;
        var plotCol = 2;
//Avoid center of star (i=0): log(r=0) undefined:
        //var minXData = log10E * logRvar[1];
        //var maxXData = log10E * logRvar[iUpper - 1];
        var minXData = rvarMm[0];
        var maxXData = rvarMm[iUpper - 1];
        //console.log("minXData " + minXData + " maxXData " + maxXData);


//var xAxisName = "Log<tspan style='baseline-shift:sub;'>10</tspan> <tspan style='font:italic;'>r</tspan><tspan style='baseline-shift:sub;'></tspan> (cm)";
        var xAxisName = "<tspan style='font:italic;'>r</tspan><tspan style='baseline-shift:sub;'></tspan> (Mm)";
        // From Hydrostat.hydrostat:
        //var numYTicks = 6;
        // Build total P from P_Gas & P_Rad:
        //var minYData = log10E * logPTot[0] - 2.0; // Avoid upper boundary condition [i]=0
        //var minYData = log10E * Math.min(logPGas[0], logPRad[0]) - 1.0;
        //var maxYData = log10E * logPTot[iUpper - 1];
        var minYData = log10E * Math.min(logPGas[iUpper - 1], logPRad[iUpper - 1]) - 1.0;
        var maxYData = log10E * logPTot[0];
        //var yAxisName = "Log<sub>10</sub> <em>P</em> <br />(dynes <br />cm<sup>-2</sup>)";
        var yAxisName = "Log<tspan style='baseline-shift:sub;'>10</tspan> <tspan style='font:italic;'>P</tspan> (dynes cm<tspan style='baseline-shift:super;'>-2</tspan>)";
        //console.log("minYData " + minYData + " maxYData " + maxYData);
        //washer(xRange, xOffset, yRange, yOffset, wDefaultColor, plotThreeId);

        washerGrid(panelWidth, panelHeight,
            wDiskColor, plotThreeId, cnvsThreeId);

        var fineness = "normal";
        //var cnvsCtx = washer(plotRow, plotCol, wDefaultColor, plotThreeId, cnvsId);
				//JB
        //var panelOrigin = washer(plotRow, plotCol, wDefaultColor, plotThreeId, cnvsThreeId);
				//JB
        //panelX = panelOrigin[0];
        //panelY = panelOrigin[1];
	//cnvsThreeId.setAttribute('fill', wDefaultColor);
//console.log("panelWidth " + panelWidth + " panelHeight " + panelHeight + " xAxisLength " + xAxisLength + " yAxisLength " + yAxisLength);
//console.log("minXData " + minXData + " maxXData " + maxXData + " minYData " + minYData + " maxYData " + maxYData + " fineness " + fineness);
        var xAxisParams = XAxis(panelWidth, panelHeight, xAxisLength, yAxisLength,
                minXData, maxXData, xAxisName, fineness,
                plotThreeId, cnvsThreeId);
        var yAxisParams = YAxis(panelWidth, panelHeight, xAxisLength, yAxisLength,
                minYData, maxYData, yAxisName,
                fineness, plotThreeId, cnvsThreeId);
        //xOffset = xAxisParams[0];
        //yOffset = xAxisParams[4];
        var rangeXData3 = xAxisParams[1];
        var deltaXData3 = xAxisParams[2];
        var deltaXPxl3 = xAxisParams[3];
        var rangeYData3 = yAxisParams[1];
        var deltaYData3 = yAxisParams[2];
        var deltaYPxl3 = yAxisParams[3];
        var xLowerYOffset3 = xAxisParams[5];
        var minXData3 = xAxisParams[6]; //updated value
        var minYData3 = yAxisParams[6]; //updated value
        var maxXData3 = xAxisParams[7]; //updated value
        var maxYData3 = yAxisParams[7]; //updated value        
//console.log("rangeXData3 " +  rangeXData3 + " deltaXData3 " + deltaXData3 + " deltaXPxl3 " + deltaXPxl3 + " rangeYData3 " + rangeYData3 + " deltaYData3 " + deltaYData3 + " deltaYPxl3 " + deltaYPxl3 + " xLowerYOffset3 " + xLowerYOffset3 + " minXData3 " + minXData3 + " minYData3 " + minYData3 + " maxXData3 " + maxXData3 + " maxYData3 " + maxYData3);
        yFinesse = 0;       
        xFinesse = 0;       
        //
        //titleX = panelX + titleOffsetX;
        //titleY = panelY + titleOffsetY;
					//JB
//        txtPrint("log Pressure: <span style='color:blue' title='Total pressure'><strong><em>P</em><sub>Tot</sub></strong></span> "
//                + " <a href='http://en.wikipedia.org/wiki/Gas_laws' target='_blank'><span style='color:#00FF88' title='Gas pressure'><em>P</em><sub>Gas</sub></span></a> "
//                + " <a href='http://en.wikipedia.org/wiki/Radiation_pressure' target='_blank'><span style='color:red' title='Radiation pressure'><em>P</em><sub>Rad</sub></span></a> " +
//                  " <span style='color:black' title='Partial electron pressure'><em>P</em><sub>e</sub></span>",
//                titleOffsetX, titleOffsetY, 300, lineColor, plotThreeId);
        txtSVGprint("log Pressure: <tspan style='fill:blue;><tspan style='font:italic;'>P<tspan style='baseline-shift:sub;'>Tot</tspan></tspan> "
                +                " <tspan style='fill:#00FF88;><tspan style='font:italic;'>P<tspan style='baseline-shift:sub;'>Gas</tspan></tspan> "
                +                " <tspan style='fill:red;><tspan style='font:italic;'>P<tspan style='baseline-shift:sub;'>Rad</tspan></tspan>", 
                titleOffsetX, titleOffsetY, lineColor, cnvsThreeId);

					//JB
        //Data loop - plot the result!

        var dSizeCnvs = 2.0; //plot point size
        var dSizeGCnvs = 1.0;
        var opac = 1.0; //opacity
        // RGB color
        var r255 = 0;
        var g255 = 0;
        var b255 = 0; //black 
        // PTot:
        var r255B = 0;
        var g255B = 0;
        var b255B = 255; //blue 
        // PGas:
        var r255G = 0;
        var g255G = 255;
        var b255G = 100; //green
        // PRad:
        var r255R = 255;
        var g255R = 0;
        var b255R = 0; //red

// Generally, plot every 10th depth:
	var lengthPlotV = Math.round(iUpper/10);
	//console.log("iUpper " + iUpper + " lengthPlotV " + lengthPlotV);
        var RGBHexV = [];
        RGBHexV.length = lengthPlotV;
        var xVec = [];
        xVec.length = lengthPlotV;
        var yVec = [];
        yVec.length = lengthPlotV;

    //Initialize plot argument arrays to valid value:
     for (var i = 0; i < lengthPlotV; i++){
        RGBHexV[i] = 0x000000;
	xVec[i] = rvarMm[0];
	yVec[i] = log10E * logPTot[0]; 
     }

////Total Pressure
//        //console.log("iUpper " + iUpper + " logRvar[iUpper-1] " + log10E*logRvar[iUpper-1] + " logPTot[iUpper-1] " + log10E*logPTot[iUpper-1]);
       j = 0;
       for (var i = 0; i < iUpper; i+=10){
           RGBHexV[j] = colHex(r255B, g255B, b255B);
           //xVec[i] = log10E * logRvar[i];
           xVec[j] = rvarMm[i];
           yVec[j] = log10E * logPTot[i];
           //if ( (i%10) == 0 ){
             //console.log("j " + j + " i " + i + " logRvar[i] " + log10E*logRvar[i] + " logPTot[i] " + log10E*logPTot[i]);
	  // }
	   j++;
        }

     plot(cnvsThreeId, xVec, yVec,
          panelWidth, panelHeight, xAxisLength, yAxisLength,
          minXData3, maxXData3, rangeXData3, minYData3, maxYData3, rangeYData3,
          RGBHexV, '-', dSizeCnvs);


//Gas Pressure
    //Initialize plot argument arrays to valid value:
     for (var i = 0; i < lengthPlotV; i++){
	yVec[i] = log10E * logPGas[0]; 
     }

        j = 0;
        for (var i = 5; i < iUpper; i+=10){
           RGBHexV[j] = colHex(r255G, g255G, b255G);
           yVec[j] = log10E * logPGas[i];
	   j++;
        }

     plot(cnvsThreeId, xVec, yVec,
          panelWidth, panelHeight, xAxisLength, yAxisLength,
          minXData3, maxXData3, rangeXData3, minYData3, maxYData3, rangeYData3,
          RGBHexV, '-', dSizeCnvs);

//Radiation pressure
    //Initialize plot argument arrays to valid value:
     for (var i = 0; i < lengthPlotV; i++){
	yVec[i] = log10E * logPRad[0]; 
     }

        j = 0;
        for (var i = 3; i < iUpper; i+=10){
           RGBHexV[j] = colHex(r255R, g255R, b255R);
           yVec[j] = log10E * logPRad[i];
	   j++;
        }

     plot(cnvsThreeId, xVec, yVec,
          panelWidth, panelHeight, xAxisLength, yAxisLength,
          minXData3, maxXData3, rangeXData3, minYData3, maxYData3, rangeYData3,
          RGBHexV, '-', dSizeCnvs);

//Comparison to standard solar model from Table 3A of 
// ApJ, 387, 372 (1992) Guenther, D.B, Demarque, P., Kim, Y-C., Pinsonneault, M.
    if (ifStandardSun){
       var pSunRef = pRef();
       var logPSunRef = [];
       logPSunRef.length = numRef;
       for (var i=0; i<numRef; i++){
          logPSunRef[i] = log10E*Math.log(pSunRef[i]);
       }
       plot(cnvsThreeId, rSunRef, logPSunRef,
          panelWidth, panelHeight, xAxisLength, yAxisLength,
          minXData3, maxXData3, rangeXData3, minYData3, maxYData3, rangeYData3,
          RGBHexSun, 'o', dSizeCnvs);
    }


// nuclear burning core radius cross-hair

        var barWidth = 1.0;
        var barColor = "#777777";
        yFinesse = 0.0;
                                //JB
        xShift = YBar(rvarMm[iCore], minXData3, maxXData3, barWidth, yAxisLength,
                panelWidth, panelHeight, xAxisLength, yAxisLength,
                yFinesse, barColor, plotThreeId, cnvsThreeId);
        barHeight = 1.0;
        yShift = XBar(log10E*logPTot[iCore], minYData3, maxYData3, xAxisLength, barHeight,
                panelWidth, panelHeight, xAxisLength, yAxisLength,
                xFinesse, barColor, plotThreeId, cnvsThreeId);
        //txtPrint("<span style='font-size:small; color:#444444'><em>&#964</em><sub>Ros</sub>=1</span>",
        //        xShift, yShift, 300, lineColor, plotThreeId);
        txtSVGprint("<tspan style='font:0.75em; fill:#444444;'><tspan style='font:italic;'>r</tspan><tspan style='baseline-shift:sub;'>Core</tspan></tspan>",
                (xShift+5), (yShift-15), lineColor, cnvsThreeId);


    //cnvsThreeId.addEventListener("mouseover", function() { 
    cnvsThreeId.addEventListener("click", function() {
       //dataCoords(event, plotThreeId);
       var xyString = dataCoords(event, cnvsThreeId, xAxisLength, minXData3, rangeXData3, xAxisXCnvs,
                               yAxisLength, minYData3, rangeYData3, yAxisYCnvs);
       //txtPrint(xyString, titleOffsetX+200, titleOffsetY+320, 150, lineColor, plotThreeId);
       txtSVGprint(xyString, titleOffsetX+200, titleOffsetY+320, lineColor, cnvsThreeId);
    }); 


				//JB
    }



//
//
//  *****   PLOT FIVE / PLOT 5
//
//

// Plot five: SED
// 
    //if ((ifLineOnly === false) && (ifShowRad === true)) { 
    //if ((ifLineOnly === false)) { 
//    //For movie:
//    if (ifLineOnly === false) { 

        var plotRow = 1;
        var plotCol = 1;
        ////For movie:
        //var plotRow = 1;
        //var plotCol = 1;
//
       //console.log("lambdaScale[0] " + lambdaScale[0] + " [1] " + lambdaScale[1]);
        //var minXData = 1.0e7 * lambdaScale[0];
        //var maxXData = 1.0e7 * lambdaScale[numLams - 1];
        //var xAxisName = "<em>&#955</em> (nm)";
            ////Logarithmic x:
        var minXData = 7.0 + logTen(lambdaScale[0]);
        var maxXData = 7.0 + logTen(lambdaScale[numLams - 1]);
        //var maxXData = 3.0; //finesse - Log10(lambda) = 3.5 nm
        //var xAxisName = "Log<sub>10</sub> &#955 (nm)";
        var xAxisName = "Log<tspan style='baseline-shift:sub;'>10</tspan> \u03bb (nm)";
        //var numYTicks = 4;
        //now done above var norm = 1.0e15; // y-axis normalization
        //var minYData = 0.0;
        //// iLamMax established in PLOT TWO above:
        //var maxYData = Blam[iLamMax] / norm;
        //var yAxisName = "<span title='Monochromatic surface flux'><a href='http://en.wikipedia.org/wiki/Spectral_flux_density' target='_blank'> <em>F</em><sub>&#955</sub> x 10<sup>15</sup><br />ergs s<sup>-1</sup> <br />cm<sup>-3</sup></a></span>";
        //Logarithmic y:
        var minYData = log10E * logBlam[iLamMin];
        var maxYData = log10E * logBlam[iLamMax];
        //var yAxisName = "<span title='Monochromatic surface flux'><a href='http://en.wikipedia.org/wiki/Spectral_flux_density' target='_blank'>Log<sub>10</sub> <em>F</em><sub>&#955</sub> <br /> ergs s<sup>-1</sup> cm<sup>-3</sup></a></span>";
        var yAxisName = "Log<tspan style='baseline-shift:sub;'>10</tspan> <tspan style='font:italic;'>B</tspan><tspan style='baseline-shift:sub;'>\u03bb</tspan> ergs s<tspan style='baseline-shift:super;'>-1</tspan> cm<tspan style='baseline-shift:super;'>-3</tspan>";
        //(xRange, xOffset, yRange, yOffset, wDefaultColor, plotFiveId);

        //var fineness = "ultrafine";
        //var cnvsCtx = washer(plotRow, plotCol, wDefaultColor, plotFiveId, cnvsId);

					//JB
        //var panelOrigin = washer(plotRow, plotCol, wDefaultColor, plotFiveId, cnvsFiveId);
					//JB

        //panelX = panelOrigin[0];
        //panelY = panelOrigin[1];
        washerGrid(panelWidth, panelHeight,
            wDiskColor, plotFiveId, cnvsFiveId);

		//console.log(SVGFive); is good, created fine
		//console.log("Before: minXData, maxXData " + minXData + ", " + maxXData);
                //console.log("XAxis called from PLOT 5:");
        var xAxisParams = XAxis(panelWidth, panelHeight, xAxisLength, yAxisLength,
                minXData, maxXData, xAxisName, fineness,
                plotFiveId, cnvsFiveId);

        var yAxisParams = YAxis(panelWidth, panelHeight, xAxisLength, yAxisLength,
                 minYData, maxYData, yAxisName, 
                 fineness, plotFiveId, cnvsFiveId);   

        //xOffset = xAxisParams[0];
        //yOffset = xAxisParams[4];
        var rangeXData5 = xAxisParams[1];
        var deltaXData5 = xAxisParams[2];
        var deltaXPxl5 = xAxisParams[3];
        var rangeYData5 = yAxisParams[1];
        var deltaYData5 = yAxisParams[2];
        var deltaYPxl5 = yAxisParams[3];
        var minXData5 = xAxisParams[6]; //updated value
        var minYData5 = yAxisParams[6]; //updated value
        var maxXData5 = xAxisParams[7]; //updated value
        var maxYData5 = yAxisParams[7]; //updated value        
		//console.log("After : minXData, maxXData " + minXData + ", " + maxXData + " rangeXData " + rangeXData);
        //
//
        // Add legend annotation:
        //titleX = panelX + titleOffsetX;
        //titleY = panelY + titleOffsetY;
					//JB
        //txtPrint("<span style='font-size:normal; color:blue'><a href='http://en.wikipedia.org/wiki/Spectral_energy_distribution' target='_blank'>\n\
     //Spectral energy distribution (SED)</a></span>",
      //          titleOffsetX, titleOffsetY, 300, lineColor, plotFiveId);
        txtSVGprint("<tspan style='fill:blue;'> Planckian spectral energy distribution (SED)</tspan>",
                titleOffsetX, titleOffsetY, lineColor, cnvsFiveId);
        //txtPrint("<span style='font-size:small'>"
        //        + "<span><em>F</em><sub>&#955</sub> (<em>&#955</em><sub>Max</sub> = " + lamMaxStr + " nm)</span>, "
        //        + " <span><em>I</em><sub>&#955</sub>,</span> <span style='color:#444444'> <em>&#952</em> = " + thet0Str + "<sup>o</sup></span>,  "
        //        + " <span style='color:#444444'><em>&#952</em> = " + thetNStr + "<sup>o</sup></span></span>",
        //        titleOffsetX, titleOffsetY+35, 250, lineColor, plotFiveId);
        txtSVGprint("<tspan style='font:0.75em;'> <tspan style='font:italic;'>I</tspan><tspan style='baseline-shift:sub;'>\u03bb</tspan> (<tspan style='font:italic;'>\u03bb</tspan><tspan style='baseline-shift:sub;'>Max</tspan> = " + lamMaxStr +' nm)', 
                titleOffsetX, titleOffsetY+35, lineColor, cnvsFiveId);
					//JB
        // Photometric bands centers

        var opac = 0.5;
        var opacStr = "0.5";
//        var yTickPos = 0;
//        var yShift = (xLowerYOffset - yRange) + yTickPos;
//        var yShiftStr = numToPxStrng(yShift);
        var vBarWidth = 2; //pixels 
        var vBarHeight = yAxisLength;
//        var vBarWidthStr = numToPxStrng(vBarWidth);
//        var vBarHeightStr = numToPxStrng(vBarHeight);
        //
        yFinesse = 0; 
        var UBVRIBands = function(r255, g255, b255, band0) {

            var RGBHex = colHex(r255, g255, b255);
            //var RGBHex = "#FF0000";
            // Vertical bar:
//            var xTickPos = xRange * (band0 - minXData) / rangeXData; // pixels    
 //           var xShift = xOffset + xTickPos;
 //  

					//JB
        xShift = YBar(band0, minXData5, maxXData5, vBarWidth, yAxisLength,
                panelWidth, panelHeight, xAxisLength, yAxisLength,
                yFinesse, RGBHex, plotFiveId, cnvsFiveId);
        }; //end function UBVRIbands
					

//
        //
        var filters = filterSet();
        var lam0_ptr = 11; // approximate band centre
        var numBands = filters.length;
        var lamUBVRI = [];
        lamUBVRI.length = numBands;

        
        for (var ib = 0; ib < numBands; ib++) {
            //lamUBVRI[ib] = 1.0e7 * filters[ib][0][lam0_ptr]; //linear lambda
            lamUBVRI[ib] = 7.0 + logTen(filters[ib][0][lam0_ptr]);  //logarithmic lambda
        }

        //Ux:
        var r255 = 155;
        var g255 = 0;
        var b255 = 155; // violet
        UBVRIBands(r255, g255, b255, lamUBVRI[0]);
        //B:
        var r255 = 0;
        var g255 = 0;
        var b255 = 255; // blue
        UBVRIBands(r255, g255, b255, lamUBVRI[2]);
        //V:
        var r255 = 0;
        var g255 = 255;
        var b255 = 100; // green
        UBVRIBands(r255, g255, b255, lamUBVRI[3]);
        //R:
        var r255 = 255;
        var g255 = 0;
        var b255 = 0; // red
        UBVRIBands(r255, g255, b255, lamUBVRI[4]);
        //I:
        var r255 = 255;
        var g255 = 40;
        var b255 = 40; // dark red / brown ??
        UBVRIBands(r255, g255, b255, lamUBVRI[5]);
	//J: (lburns)
	var r255 = 178;
	var g255 = 34;
	var b255 = 34; // firebrick 
	UBVRIBands(r255, g255, b255, lamUBVRI[6]);
	//H: (lburns)
	var r255 = 128;
	var g255 = 0;
	var b255 = 0; // maroon
	UBVRIBands(r255, g255, b255, lamUBVRI[7]);
	//K: (lburns)
	var r255 = 160;
	var g255 = 82;
	var b255 = 45; // sienna
	UBVRIBands(r255, g255, b255, lamUBVRI[8]);
        //Data loop - plot the result!

//Continuum spectrum - For testing: 
//        var contFlux3 = interpolV(contFlux[0], lambdaScale, lambdaScalemasterLams);

        var dSizeCnvs = 1.0; //plot point size
        var dSize0Cnvs = 1.0;
        var opac = 1.0; //opacity
        // RGB color
        // PTot:
        var r255 = 0;
        var g255 = 0;
        var b255 = 0; //black
        // PGas:
        var r2550 = 90;
        var g2550 = 90;
        var b2550 = 90; //dark gray
        // PRad:
        var r255N = 120;
        var g255N = 120;
        var b255N = 120; //light gray

        var RGBHexV = [];
        RGBHexV.length = numLams;
        var logLambdanm = [];
        logLambdanm.length = numLams;
        var yVec = [];
        yVec.length = numLams;

//Intensity
        for (var i = 0; i < numLams; i++){
           RGBHexV[i] = colHex(r255, g255, b255);
           logLambdanm[i] = 7.0 + logTen(lambdaScale[i]);
           yVec[i] = log10E*logBlam[i];
        }

     plot(cnvsFiveId, logLambdanm, yVec,
          panelWidth, panelHeight, xAxisLength, yAxisLength,
          minXData5, maxXData5, rangeXData5, minYData5, maxYData5, rangeYData5,
          RGBHexV, '-', dSizeCnvs);


    //cnvsFiveId.addEventListener("mouseover", function() { 
    cnvsFiveId.addEventListener("click", function() {
       //dataCoords(event, plotFiveId);
       var xyString = dataCoords(event, cnvsFiveId, xAxisLength, minXData5, rangeXData5, xAxisXCnvs,
                               yAxisLength, minYData5, rangeYData5, yAxisYCnvs);
       //txtPrint(xyString, titleOffsetX+200, titleOffsetY+320, 150, lineColor, plotFiveId);
       txtSVGprint(xyString, titleOffsetX+200, titleOffsetY+320, lineColor, cnvsFiveId);
    });

        //monochromatic disk lambda
           var yFinesse = 0.0;
           var barHeight = 200;
           var barWidth = 2;
           RGBHex = "#000000";
		//JB

   //linear wavelength
          //var xShiftDum = YBar(diskLambda, minXData5, maxXData5, barWidth, barHeight,
          //      yFinesse, RGBHex, plotFiveId, cnvsFiveId);

					//JB    
//} 


//
//
//  *****   PLOT TWO / PLOT 2
//
//

// Plot two: r vs Temp
// 
    if ((ifShowStruc === true)) {

        var plotRow = 2;
        var plotCol = 2;
        var minXData = rvarMm[0];
        var maxXData = rvarMm[iUpper-1];
	//console.log("minXData " + minXData + " maxXData " + maxXData);
        //var xAxisName = "<span title='Rosseland mean optical depth'><a href='http://en.wikipedia.org/wiki/Optical_depth_%28astrophysics%29' target='_blank'>Log<sub>10</sub> <em>&#964</em><sub>Ros</sub></a></span>";
        var xAxisName = "<tspan style='font:italic;'>r</tspan><tspan style='baseline-shift:sub;'></tspan> (Mm)";
        var minYData = log10E*logTemp[iUpper - 1];
        var maxYData = log10E*logTemp[0];
	//console.log("minYData " + minYData + " maxYData " + maxYData);
        //var yAxisName = "<em>T</em><sub>Kin</sub> (K)";
        var yAxisName = "Log<tspan style='baseline-shift:sub;'>10</tspan> <tspan style='font:italic;'>T</tspan><tspan style='baseline-shift:sub;'>Kin</tspan> (K)";
        var fineness = "normal";
					//JB
        //var panelOrigin = washer(plotRow, plotCol, wDefaultColor, plotTwoId, cnvsTwoId);
					//JB
        //panelX = panelOrigin[0];
        //panelY = panelOrigin[1];
        washerGrid(panelWidth, panelHeight,
            wDiskColor, plotTwoId, cnvsTwoId);
					//JB
	//cnvsTwoId.setAttribute('fill', wDefaultColor);
        var xAxisParams = XAxis(panelWidth, panelHeight, xAxisLength, yAxisLength,
                minXData, maxXData, xAxisName, fineness,
                plotTwoId, cnvsTwoId);
        var cnvsCtx = xAxisParams[8];
        var yAxisParams = YAxis(panelWidth, panelHeight, xAxisLength, yAxisLength, 
                minYData, maxYData, yAxisName,fineness, plotTwoId, cnvsTwoId);
                                                
					//JB

        //
        //xOffset = xAxisParams[0];
        //yOffset = xAxisParams[4];
        var rangeXData2 = xAxisParams[1];
        var deltaXData2 = xAxisParams[2];
        var deltaXPxl2 = xAxisParams[3];
        var rangeYData2 = yAxisParams[1];
        var deltaYData2 = yAxisParams[2];
        var deltaYPxl2 = yAxisParams[3];
        //var xLowerYOffset = xAxisParams[5];
        var minXData2 = xAxisParams[6]; //updated value
        var minYData2 = yAxisParams[6]; //updated value
        var maxXData2 = xAxisParams[7]; //updated value
        var maxYData2 = yAxisParams[7]; //updated value    
        yFinesse = 0;       
        xFinesse = 0;       
        //
        //txtPrint("<span style='font-size:normal; color:blue'>Gas temperature </span>",
        //        titleOffsetX, titleOffsetY, 300, lineColor, plotTwoId);
        txtSVGprint("<tspan style='fill:blue;'>Gas temperature </tspan>",
                titleOffsetX, titleOffsetY, lineColor, cnvsTwoId);
					//JB
        //var dSize = 5.0; //plot point size
        var dSizeCnvs = 1.0; //plot point size
        var opac = 1.0; //opacity
        // RGB color
        var r255 = 0;
        var g255 = 0;
        var b255 = 255; //blue

// Generally, plot every 10th depth:
	var lengthPlotV = Math.round(iUpper/10);
	//console.log("iUpper " + iUpper + " lengthPlotV " + lengthPlotV);
        var RGBHexV = [];
        RGBHexV.length = lengthPlotV;
        var xVec = [];
        xVec.length = lengthPlotV;
        var yVec = [];
        yVec.length = lengthPlotV;

    //Initialize plot argument arrays to valid value:
     for (var i = 0; i < lengthPlotV; i++){
        RGBHexV[i] = 0x000000;
	xVec[i] = rvarMm[0];
	yVec[i] = log10E*logTemp[0]; 
     }

//        console.log("iUpper " + iUpper + " logRvar[iUpper-1] " + log10E*logRvar[iUpper-1] + " logRho[iUpper-1] " + log10E*logRho[iUpper-1]);
       j = 0;
       for (var i = 0; i < iUpper; i+=10){
           RGBHexV[j] = colHex(r255B, g255B, b255B);
           //xVec[i] = log10E * logRvar[i];
           xVec[j] = rvarMm[i];
           yVec[j] = log10E * logTemp[i];
           //if ( (i%10) == 0 ){
           //  console.log("j " + j + " i " + i + " logRvar[i] " + log10E*logRvar[i] + " logTemp " + log10E*logTemp[i]);
	  // }
	   j++;
        }


     plot(cnvsTwoId, xVec, yVec,
          panelWidth, panelHeight, xAxisLength, yAxisLength,
          minXData2, maxXData2, rangeXData2, minYData2, maxYData2, rangeYData2,
          RGBHexV, '-', dSizeCnvs);


//Comparison to standard solar model from Table 3A of 
// ApJ, 387, 372 (1992) Guenther, D.B, Demarque, P., Kim, Y-C., Pinsonneault, M.
    if (ifStandardSun){
       var tSunRef = tRef();
       var logTSunRef = [];
       logTSunRef.length = numRef;
       for (var i=0; i<numRef; i++){
          logTSunRef[i] = log10E*Math.log(tSunRef[i]);
       }
       plot(cnvsTwoId, rSunRef, logTSunRef,
          panelWidth, panelHeight, xAxisLength, yAxisLength,
          minXData2, maxXData2, rangeXData2, minYData2, maxYData2, rangeYData2,
          RGBHexSun, 'o', dSizeCnvs);
    }


// nuclear burning core radius cross-hair

        var barWidth = 1.0;
        var barColor = "#777777";
        yFinesse = 0.0;
                                //JB
        xShift = YBar(rvarMm[iCore], minXData2, maxXData2, barWidth, yAxisLength,
                panelWidth, panelHeight, xAxisLength, yAxisLength,
                yFinesse, barColor, plotTwoId, cnvsTwoId);
        barHeight = 1.0;
        yShift = XBar(log10E*logTemp[iCore], minYData2, maxYData2, xAxisLength, barHeight,
                panelWidth, panelHeight, xAxisLength, yAxisLength,
                xFinesse, barColor, plotTwoId, cnvsTwoId);
        //txtPrint("<span style='font-size:small; color:#444444'><em>&#964</em><sub>Ros</sub>=1</span>",
        //        xShift, yShift, 300, lineColor, plotTwoId);
        txtSVGprint("<tspan style='font:0.75em; fill:#444444;'><tspan style='font:italic;'>r</tspan><tspan style='baseline-shift:sub;'>Core</tspan></tspan>",
                (xShift+5), (yShift-15), lineColor, cnvsTwoId);


    cnvsTwoId.addEventListener("click", function() {
       //dataCoords(event, plotTwoId);
       var xyString = dataCoords(event, cnvsTwoId, xAxisLength, minXData2, rangeXData2, xAxisXCnvs,
                               yAxisLength, minYData2, rangeYData2, yAxisYCnvs);
       //console.log("PLOT TWO INSIDE: xAxisLength " + xAxisLength + " minXData " + minXData2 + " rangeXData " + rangeXData2 + " xAxisXCnvs " + xAxisXCnvs2);
       //txtPrint(xyString, titleOffsetX+200, titleOffsetY+320, 150, lineColor, plotTwoId);
       txtSVGprint(xyString, titleOffsetX+200, titleOffsetY+320, lineColor, cnvsTwoId);
    });


    } //End plot two


//
//
//  *****   PLOT FOUR / PLOT 4
//
//

// Plot four: r vs cumulative luminosity (lvar) 
// 
    if ((ifShowStruc === true)) {

        var plotRow = 4;
        var plotCol = 2;
        var minXData = rvarMm[0];
        var maxXData = rvarMm[iUpper-1];
	//console.log("minXData " + minXData + " maxXData " + maxXData);
        //var xAxisName = "<span title='Rosseland mean optical depth'><a href='http://en.wikipedia.org/wiki/Optical_depth_%28astrophysics%29' target='_blank'>Log<sub>10</sub> <em>&#964</em><sub>Ros</sub></a></span>";
        var xAxisName = "<tspan style='font:italic;'>r</tspan><tspan style='baseline-shift:sub;'></tspan> (Mm)";
        var minYData = log10E*logLvar[2];
        var maxYData = log10E*logLvar[iUpper-1];
	//console.log("minYData " + minYData + " maxYData " + maxYData);
        //var yAxisName = "<em>T</em><sub>Kin</sub> (K)";
        var yAxisName = "Log<tspan style='baseline-shift:sub;'>10</tspan> <tspan style='font:italic;'>L</tspan><tspan style='baseline-shift:sub;'>r</tspan> (erg/s)";
        var fineness = "normal";
					//JB
        //var panelOrigin = washer(plotRow, plotCol, wDefaultColor, plotFourId, cnvsFourId);
					//JB
        //panelX = panelOrigin[0];
        //panelY = panelOrigin[1];
        washerGrid(panelWidth, panelHeight,
            wDiskColor, plotFourId, cnvsFourId);
					//JB
	//cnvsFourId.setAttribute('fill', wDefaultColor);
        var xAxisParams = XAxis(panelWidth, panelHeight, xAxisLength, yAxisLength,
                minXData, maxXData, xAxisName, fineness,
                plotFourId, cnvsFourId);
        var cnvsCtx = xAxisParams[8];
        var yAxisParams = YAxis(panelWidth, panelHeight, xAxisLength, yAxisLength, 
                minYData, maxYData, yAxisName,fineness, plotFourId, cnvsFourId);
                                                
					//JB

        //
        //xOffset = xAxisParams[0];
        //yOffset = xAxisParams[4];
        var rangeXData4 = xAxisParams[1];
        var deltaXData4 = xAxisParams[2];
        var deltaXPxl4 = xAxisParams[3];
        var rangeYData4 = yAxisParams[1];
        var deltaYData4 = yAxisParams[2];
        var deltaYPxl4 = yAxisParams[3];
        //var xLowerYOffset = xAxisParams[5];
        var minXData4 = xAxisParams[6]; //updated value
        var minYData4 = yAxisParams[6]; //updated value
        var maxXData4 = xAxisParams[7]; //updated value
        var maxYData4 = yAxisParams[7]; //updated value    
        yFinesse = 0;       
        xFinesse = 0;       
        //
        //txtPrint("<span style='font-size:normal; color:blue'>Gas temperature </span>",
        //        titleOffsetX, titleOffsetY, 300, lineColor, plotFourId);
        txtSVGprint("<tspan style='fill:blue;'>Cumulative luminosity</tspan>",
                titleOffsetX, titleOffsetY, lineColor, cnvsFourId);
					//JB
        //var dSize = 5.0; //plot point size
        var dSizeCnvs = 1.0; //plot point size
        var opac = 1.0; //opacity
        // RGB color
        var r255 = 0;
        var g255 = 0;
        var b255 = 255; //blue

// Generally, plot every 10th depth:
	var lengthPlotV = Math.round(iUpper/10);
	//console.log("iUpper " + iUpper + " lengthPlotV " + lengthPlotV);
        var RGBHexV = [];
        RGBHexV.length = lengthPlotV;
        var xVec = [];
        xVec.length = lengthPlotV;
        var yVec = [];
        yVec.length = lengthPlotV;

    //Initialize plot argument arrays to valid value:
     for (var i = 0; i < lengthPlotV; i++){
        RGBHexV[i] = 0x000000;
	xVec[i] = rvarMm[0];
	yVec[i] = log10E*logLvar[2]; 
     }

//        console.log("iUpper " + iUpper + " logRvar[iUpper-1] " + log10E*logRvar[iUpper-1] + " logRho[iUpper-1] " + log10E*logRho[iUpper-1]);
       j = 0;
       for (var i = 0; i < iUpper; i+=10){
           RGBHexV[j] = colHex(r255B, g255B, b255B);
           //xVec[i] = log10E * logRvar[i];
           xVec[j] = rvarMm[i];
           yVec[j] = log10E * logLvar[i];
           //if ( (i%10) == 0 ){
           //  console.log("j " + j + " i " + i + " logRvar[i] " + log10E*logRvar[i] + " logTemp " + log10E*logTemp[i]);
	  // }
	   j++;
        }


     plot(cnvsFourId, xVec, yVec,
          panelWidth, panelHeight, xAxisLength, yAxisLength,
          minXData4, maxXData4, rangeXData4, minYData4, maxYData4, rangeYData4,
          RGBHexV, '-', dSizeCnvs);


//Comparison to standard solar model from Table 3A of 
// ApJ, 387, 372 (1992) Guenther, D.B, Demarque, P., Kim, Y-C., Pinsonneault, M.
    if (ifStandardSun){
       var lSunRef = lRef();
       var logLSunRef = [];
       logLSunRef.length = numRef;
       for (var i=0; i<numRef; i++){
          logLSunRef[i] = log10E*Math.log(lSunRef[i]);
       }
       plot(cnvsFourId, rSunRef, logLSunRef,
          panelWidth, panelHeight, xAxisLength, yAxisLength,
          minXData4, maxXData4, rangeXData4, minYData4, maxYData4, rangeYData4,
          RGBHexSun, 'o', dSizeCnvs);
    }


// nuclear burning core radius cross-hair

        var barWidth = 1.0;
        var barColor = "#777777";
        yFinesse = 0.0;
                                //JB
        xShift = YBar(rvarMm[iCore], minXData4, maxXData4, barWidth, yAxisLength,
                panelWidth, panelHeight, xAxisLength, yAxisLength,
                yFinesse, barColor, plotFourId, cnvsFourId);
        barHeight = 1.0;
        yShift = XBar(log10E*logLvar[iCore], minYData4, maxYData4, xAxisLength, barHeight,
                panelWidth, panelHeight, xAxisLength, yAxisLength,
                xFinesse, barColor, plotFourId, cnvsFourId);
        //txtPrint("<span style='font-size:small; color:#444444'><em>&#964</em><sub>Ros</sub>=1</span>",
        //        xShift, yShift, 300, lineColor, plotFourId);
        txtSVGprint("<tspan style='font:0.75em; fill:#444444;'><tspan style='font:italic;'>r</tspan><tspan style='baseline-shift:sub;'>Core</tspan></tspan>",
                (xShift+5), (yShift-15), lineColor, cnvsFourId);


    cnvsFourId.addEventListener("click", function() {
       //dataCoords(event, plotFourId);
       var xyString = dataCoords(event, cnvsFourId, xAxisLength, minXData4, rangeXData4, xAxisXCnvs,
                               yAxisLength, minYData4, rangeYData4, yAxisYCnvs);
       //console.log("PLOT TWO INSIDE: xAxisLength " + xAxisLength + " minXData " + minXData4 + " rangeXData " + rangeXData4 + " xAxisXCnvs " + xAxisXCnvs4);
       //txtPrint(xyString, titleOffsetX+400, titleOffsetY+340, 150, lineColor, plotFourId);
       txtSVGprint(xyString, titleOffsetX+200, titleOffsetY+320, lineColor, cnvsFourId);
    });


  } //End plot four 

					
//
//
//  *****   PLOT ELEVEN / PLOT 11
//
//
// Plot Eleven: Life Zone


        var plotRow = 0;
        var plotCol = 2;

//background color needs to be finessed so that white-ish stars will stand out:
       if (teff > 6000.0){
  //hotter white or blue-white star - darken the background (default background in #F0F0F0
           wDiskColor =  wDefaultColor;  
       } else {
           wDiskColor = wDefaultColor;
       }

        // Calculation of steam line and ice line:

        //Assuming liquid salt-free water at one atmospheric pGasressure is necessary:
        //var atmosPres = 101.0;  // test - kPa
//        var steamTemp = waterPhase(atmosPress);
       var steamTemp = solventPhase(atmosPress, phaseA, phaseB, phaseC);
        //console.log("steamTemp " + steamTemp); // + " steamTemp2 " + steamTemp2);
        //var steamTemp = 373.0; // K = 100 C
        //var iceTemp = 273.0; //K = 0 C
        var iceTemp = tripleTemp; 

        steamTemp = steamTemp - greenHouse;
        iceTemp = iceTemp - greenHouse;
        var logSteamLine, logIceLine;
        //var au = 1.4960e13; // 1 AU in cm
        //var rSun = 6.955e10; // solar radii to cm
        var log1AULine = logAu - logRSun; // 1 AU in solar radii
        //Steam line:
        //Set steamTemp equal to planetary surface temp and find distance that balances stellar irradiance 
        //absorbed by planetary cross-section with planet's bolometric thermal emission:
        //Everything in solar units -> distance, d, in solar radii
        logSteamLine = 2.0 * (Math.log(teff) - Math.log(steamTemp)) + logRadius + 0.5 * Math.log(1.0 - albedo);
        //now the same for the ice line:
        logIceLine = 2.0 * (Math.log(teff) - Math.log(iceTemp)) + logRadius + 0.5 * Math.log(1.0 - albedo);
        var iceLineAU = Math.exp(logIceLine) * rSun / au;
        var steamLineAU = Math.exp(logSteamLine) * rSun / au;
        iceLineAU = iceLineAU.toPrecision(3);
        steamLineAU = steamLineAU.toPrecision(3);
        var steamTempRound = steamTemp.toPrecision(3);
        //console.log(" logSteamLine " + logSteamLine + " logIceLine " + logIceLine); 

        // Convert solar radii to pixels:

        var radiusScale = 20; //solar_radii-to-pixels!
        var logScale = 20; //amplification factor for log pixels

        // 
        // Star radius in pixels:

        //    var radiusPx = (radiusScale * radius);  //linear radius
        var radiusPx = logScale * logTen(radiusScale * radius); //logarithmic radius
        radiusPx = Math.ceil(radiusPx);
        var radiusPxSteam = logScale * logTen(radiusScale * radius * Math.exp(logSteamLine));
        radiusPxSteam = Math.ceil(radiusPxSteam);
        var radiusPxIce = logScale * logTen(radiusScale * radius * Math.exp(logIceLine));
        radiusPxIce = Math.ceil(radiusPxIce);
        var radiusPx1AU = logScale * logTen(radiusScale * radius * Math.exp(log1AULine));
        radiusPx1AU = Math.ceil(radiusPx1AU);
 
    //console.log("radius " + radius + " radiusPx " + radiusPx + " radiusPxSteam " + radiusPxSteam + " radiusPxIce " + radiusPxIce + " radiusPx1AU " + radiusPx1AU)
        // Key radii in order of *DECREASING* size (important!):
        //var numZone = 7;
        //var radii = [];
        //radii.length = numZone;
// Safety defaults:
        //radii = [radiusPx1AU, radiusPx1AU, radiusPx1AU, radiusPx1AU, radiusPx1AU, radiusPx1AU, radiusPx1AU]
        rrI = saveRGB[0];
        ggI = saveRGB[1];
        bbI = saveRGB[2];
        var starRGBHex = "rgb(" + rrI + "," + ggI + "," + bbI + ")";
        //var colors = [];
        //colors.length = numZone;

        //radii = [radiusPx1AU+1, radiusPx1AU, radiusPxIce + 3, radiusPxIce, radiusPxSteam, radiusPxSteam - 3, radiusPx];
        var lzWdthPx = radiusPxIce - radiusPxSteam; //life zone width in pixels
        lzWdthPx = Math.round(lzWdthPx);
        var lzRdsPx = ( radiusPxSteam + radiusPxIce ) / 2.0;
        lzRdsPx = Math.round(lzRdsPx);

     //console.log("radii " + radii)
        //
        //var titleYPos = xLowerYOffset - yRange + 40;
					//JB
      //var panelOrigin = washer(plotRow, plotCol, wDiskColor, plotElevenId, cnvsElevenId);
					//JB
	//panelX = panelOrigin[0];
        //panelY = panelOrigin[1];
        washerGrid(panelWidth, panelHeight,
            wDiskColor, plotElevenId, cnvsElevenId);
	//cnvsElevenId.setAttribute('fill', wDiskColor);
        // Add title annotation:

        //titleX = panelX + titleOffsetX;
        //titleY = panelY + titleOffsetY;
			//JB
      
        //txtPrint("<span style='font-size:normal; color:blue' title='Assumes liquid salt-free water at one Earth atmosphere pressure needed for life'><a href='https://en.wikipedia.org/wiki/Circumstellar_habitable_zone' target='_blank'>Life zone for habitable planets</a></span><br />\n\
     //<span style='font-size:small'>(Logarithmic radius)</span>",
      //          titleOffsetX, titleOffsetY, 300, lineColor, plotElevenId);
        txtSVGprint("<tspan style='fill:blue;'>Life zone for habitable planets</tspan> <tspan style='font:0.75em;'>(Logarithmic radius)</tspan>",
                titleOffsetX, titleOffsetY, lineColor, cnvsElevenId);
        var legendY = titleOffsetY;
        var legendX = titleOffsetX + 320;
        //txtPrint("<span style='font-size:small'>"
        //        + " <span style='color:#FF0000'>Steam line</span> " + steamLineAU + " <a href='https://en.wikipedia.org/wiki/Astronomical_unit' title='1 AU = Earths average distance from center of Sun'> AU</a><br /> "
        //        + " <span style='color:#00FF88'><strong>Life zone</strong></span><br /> "
        //        + " <span style='color:#0000FF'>Ice line</span> " + iceLineAU + " <a href='https://en.wikipedia.org/wiki/Astronomical_unit' title='1 AU = Earths average distance from center of Sun'> AU</a><br /> " 
        //        + " <span style='color:#000000'>Reference line: 1 <a href='https://en.wikipedia.org/wiki/Astronomical_unit' title='1 AU = Earths average distance from center of Sun'>AU</a></span>",
        //        legendX, legendY, 300, lineColor, plotElevenId);
//        txtSVGprint("<tspan style='font:0.75em;fll:#FF0000'>Steam line</tspan> " + steamLineAU 
//              + " AU <tspan style='font:bold; fill:#00FF88;'>Life zone</tspan> <tspan style='fill:#0000FF;'>Ice line</tspan> " 
//              + iceLineAU + " AU " 
//              + rOrbitAU + " AU <tspan style='fill:#000000';>Reference line: 1>AU</tspan>",
//                legendX, legendY, lineColor, cnvsElevenId);
        txtSVGprint("<tspan style='font-size:0.8em; fill:red;'>Steam line " + steamLineAU + " AU</tspan>", legendX, legendY+20, "#FF0000", cnvsElevenId);
        txtSVGprint("<tspan style='font-size:0.8em; fill:blue;'>Ice line " + iceLineAU + " AU</tspan>", legendX, legendY+40, "#0000FF", cnvsElevenId);
        txtSVGprint("<tspan style='font-size:0.8em;'>1 AU </tspan>", legendX, legendY+60, "#000000", cnvsElevenId);
//
        //txtPrint("<span style='font-size:small'>" + solvent + " boiling temp = " + steamTempRound + " K</span>", 
        //  (legendX-75), (legendY+300), 300, lineColor, plotElevenId);
        txtSVGprint("<tspan style='font:0.75em'>" + solvent + " boiling temp = " + steamTempRound + " K</tspan>", 
          (legendX-75), (legendY+300), lineColor, cnvsElevenId);
        //Get the Vega-calibrated colors from the intensity spectrum of each theta annulus:    
        // moved earlier var intcolors = iColors(lambdaScale, intens, numDeps, numThetas, numLams, tauRos, temp);
			//JB

// Adjust position to center star:
// Radius is really the *diameter* of the symbol
            var yCenterCnvs = panelHeight / 2; 
            var xCenterCnvs = panelWidth / 2; 
        //  Loop over radial zones - largest to smallest
   //console.log("cx " + xCenterCnvs + " cy " + yCenterCnvs);

//Add host star:

        makeCirc(cnvsElevenId, xCenterCnvs, yCenterCnvs, radiusPx,
                 starRGBHex, 2, 1, starRGBHex, 1);
	
//Add life zone:

        makeCirc(cnvsElevenId, xCenterCnvs, yCenterCnvs, lzRdsPx,
                 "#00FF88", lzWdthPx, 1, "#FFFFFF", 0);
	
//Add steam line:

        makeCirc(cnvsElevenId, xCenterCnvs, yCenterCnvs, radiusPxSteam,
                 "#FF0000", 2, 1, "#FFFFFF", 0);
	
				
//Add ice line:

        makeCirc(cnvsElevenId, xCenterCnvs, yCenterCnvs, radiusPxIce,
                 "#0000FF", 2, 1, "#FFFFFF", 0);
	
				
//Add 1 AU line:

        makeCirc(cnvsElevenId, xCenterCnvs, yCenterCnvs, radiusPx1AU,
                 "#000000", 2, 1, "#FFFFFF", 0);
	
				
					

// ****************************************
    //
    //
    //  *****   PLOT FOURTEEN / PLOT 14
    //

// Plot fourteen: r vs opacity (kappa) 
// 
    if ((ifShowStruc === true)) {

        var plotRow = 4;
        var plotCol = 2;
        var minXData = rvarMm[0];
        var maxXData = rvarMm[iUpper-1];
	//console.log("minXData " + minXData + " maxXData " + maxXData);
        //var xAxisName = "<span title='Rosseland mean optical depth'><a href='http://en.wikipedia.org/wiki/Optical_depth_%28astrophysics%29' target='_blank'>Log<sub>10</sub> <em>&#964</em><sub>Ros</sub></a></span>";
        var xAxisName = "<tspan style='font:italic;'>r</tspan><tspan style='baseline-shift:sub;'></tspan> (Mm)";
        var minYData = log10E*Math.log(kappa[1]/2.0);
        var maxYData = log10E*Math.log(kappa[iUpper-1]);
	//console.log("14: minYData " + minYData + " maxYData " + maxYData);
        //var yAxisName = "<em>T</em><sub>Kin</sub> (K)";
        var yAxisName = "Log<tspan style='baseline-shift:sub;'>10</tspan> <tspan style='font:italic;'>\u03BA</tspan><tspan style='baseline-shift:sub;'>r</tspan> (cm^2/g)";
        var fineness = "normal";
					//JB
        //var panelOrigin = washer(plotRow, plotCol, wDefaultColor, plotFourteenId, cnvsFourteenId);
					//JB
        //panelX = panelOrigin[0];
        //panelY = panelOrigin[1];
        washerGrid(panelWidth, panelHeight,
            wDiskColor, plotFourteenId, cnvsFourteenId);
					//JB
	//cnvsFourteenId.setAttribute('fill', wDefaultColor);
        var xAxisParams = XAxis(panelWidth, panelHeight, xAxisLength, yAxisLength,
                minXData, maxXData, xAxisName, fineness,
                plotFourteenId, cnvsFourteenId);
        var cnvsCtx = xAxisParams[8];
        var yAxisParams = YAxis(panelWidth, panelHeight, xAxisLength, yAxisLength, 
                minYData, maxYData, yAxisName,fineness, plotFourteenId, cnvsFourteenId);
                                                
					//JB

        //
        //xOffset = xAxisParams[0];
        //yOffset = xAxisParams[4];
        var rangeXData14 = xAxisParams[1];
        var deltaXData14 = xAxisParams[2];
        var deltaXPxl14 = xAxisParams[3];
        var rangeYData14 = yAxisParams[1];
        var deltaYData14 = yAxisParams[2];
        var deltaYPxl14 = yAxisParams[3];
        //var xLowerYOffset = xAxisParams[5];
        var minXData14 = xAxisParams[6]; //updated value
        var minYData14 = yAxisParams[6]; //updated value
        var maxXData14 = xAxisParams[7]; //updated value
        var maxYData14 = yAxisParams[7]; //updated value    
        yFinesse = 0;       
        xFinesse = 0;       
        //
        //txtPrint("<span style='font-size:normal; color:blue'>Gas temperature </span>",
        //        titleOffsetX, titleOffsetY, 300, lineColor, plotFourteenId);
        txtSVGprint("log Mean opacity: <tspan style='fill:black;><tspan style='font:italic;'>\u03BA<tspan style='baseline-shift:sub;'>Tot</tspan></tspan> "
                +                " <tspan style='fill:blue;><tspan style='font:italic;'>\u03BA<tspan style='baseline-shift:sub;'>bf</tspan></tspan> "
                +                " <tspan style='fill:purple;><tspan style='font:italic;'>\u03BA<tspan style='baseline-shift:sub;'>ff</tspan></tspan> "
                +                " <tspan style='fill:orange;><tspan style='font:italic;'>\u03BA<tspan style='baseline-shift:sub;'>e-</tspan></tspan> "
                +                " <tspan style='fill:red;><tspan style='font:italic;'>\u03BA<tspan style='baseline-shift:sub;'>H-</tspan></tspan>", 
                titleOffsetX, titleOffsetY, lineColor, cnvsFourteenId);
					//JB
        //var dSize = 5.0; //plot point size
        var dSizeCnvs = 1.0; //plot point size
        var opac = 1.0; //opacity
        // RGB color
        var r255 = 0;
        var g255 = 0;
        var b255 = 255; //blue

// Generally, plot every 10th depth:
	var lengthPlotV = Math.round(iUpper/10);
	//console.log("iUpper " + iUpper + " lengthPlotV " + lengthPlotV);
        var RGBHexV = [];
        RGBHexV.length = lengthPlotV;
        var RGBHexVbf = [];
        RGBHexVbf.length = lengthPlotV;
        var RGBHexVff = [];
        RGBHexVff.length = lengthPlotV;
        var RGBHexVEs = [];
        RGBHexVEs.length = lengthPlotV;
        var RGBHexVHmin = [];
        RGBHexVHmin.length = lengthPlotV;
        var xVec = [];
        xVec.length = lengthPlotV;
        var yVec = [];
        yVec.length = lengthPlotV;
        var yVecbf = [];
        yVecbf.length = lengthPlotV;
        var yVecff = [];
        yVecff.length = lengthPlotV;
        var yVecEs = [];
        yVecEs.length = lengthPlotV;
        var yVecHmin = [];
        yVecHmin.length = lengthPlotV;

    //Initialize plot argument arrays to valid value:
     for (var i = 0; i < lengthPlotV; i++){
        RGBHexV[i] = 0x000000;
        RGBHexVbf[i] = 0x000000;
        RGBHexVff[i] = 0x000000;
        RGBHexVEs[i] = 0x000000;
        RGBHexVHmin[i] = 0x000000;
	xVec[i] = rvarMm[0];
        if (kappa[iUpper-1] > 0.0){
	    yVec[i] = log10E*Math.log(kappa[iUpper-1]);
	} else {
	    yVec[i] = logTiny;
	}
        if (kapBF[iUpper-1] > 0.0){
	    yVecbf[i] = log10E*Math.log(kapBF[iUpper-1]); 
	} else {
	    yVecbf[i] = logTiny;
	}
        if (kapFF[iUpper-1] > 0.0){
	    yVecff[i] = log10E*Math.log(kapFF[iUpper-1]); 
	} else {
	    yVecff[i] = logTiny;
	}
        if (kapEs[iUpper-1] > 0.0){
	    yVecEs[i] = log10E*Math.log(kapEs[iUpper-1]); 
	} else {
	    yVecEs[i] = logTiny;
	}
        if (kapHmin[iUpper-1] > 0.0){
	    yVecHmin[i] = log10E*Math.log(kapHmin[iUpper-1]); 
	} else {
	    yVecHmin[i] = logTiny;
	}
     }

//        console.log("iUpper " + iUpper + " logRvar[iUpper-1] " + log10E*logRvar[iUpper-1] + " logRho[iUpper-1] " + log10E*logRho[iUpper-1]);
       j = 0;
       for (var i = 0; i < iUpper; i+=10){
           RGBHexV[j] = colHex(r255Blck, g255Blck, b255Blck);
           RGBHexVbf[i] = colHex(r255B, g255B, b255B); 
           RGBHexVff[i] = colHex(r255P, g255P, b255P); 
           RGBHexVEs[i] = colHex(r255O, g255O, b255O); 
           RGBHexVHmin[i] = colHex(r255R, g255R, b255R); 
           //xVec[i] = log10E * logRvar[i];
           xVec[j] = rvarMm[i];
           if (kappa[i] > 0.0){
              yVec[j] = log10E * Math.log(kappa[i]);
	   } else {
	       yVec[j] = logTiny;
           }
           if (kapBF[i] > 0.0){
              yVecbf[j] = log10E * Math.log(kapBF[i]);
           } else {
	      yVecbf[j] = logTiny;
           }
           if (kapFF[i] > 0.0){
              yVecff[j] = log10E * Math.log(kapFF[i]);
           } else {
	      yVecff[j] = logTiny;
           }
           if (kapEs[i] > 0.0){
              yVecEs[j] = log10E * Math.log(kapEs[i]);
           } else {
	      yVecEs[j] = logTiny;
           }
           if (kapHmin[i] > 0.0){
              yVecHmin[j] = log10E * Math.log(kapHmin[i]);
           } else {
	      yVecHmin[j] = logTiny;
	}
           //if ( (i%10) == 0 ){
           //  console.log("j " + j + " i " + i + " logRvar[i] " + log10E*logRvar[i] + " logTemp " + log10E*logTemp[i]);
	  // }
	   j++;
        }


     plot(cnvsFourteenId, xVec, yVec,
          panelWidth, panelHeight, xAxisLength, yAxisLength,
          minXData14, maxXData14, rangeXData14, minYData14, maxYData14, rangeYData14,
          RGBHexV, 'o', dSizeCnvs);
     plot(cnvsFourteenId, xVec, yVecbf,
          panelWidth, panelHeight, xAxisLength, yAxisLength,
          minXData14, maxXData14, rangeXData14, minYData14, maxYData14, rangeYData14,
          RGBHexVbf, '-', dSizeCnvs);
     plot(cnvsFourteenId, xVec, yVecff,
          panelWidth, panelHeight, xAxisLength, yAxisLength,
          minXData14, maxXData14, rangeXData14, minYData14, maxYData14, rangeYData14,
          RGBHexVff, '-', dSizeCnvs);
     plot(cnvsFourteenId, xVec, yVecEs,
          panelWidth, panelHeight, xAxisLength, yAxisLength,
          minXData14, maxXData14, rangeXData14, minYData14, maxYData14, rangeYData14,
          RGBHexVEs, '-', dSizeCnvs);
     plot(cnvsFourteenId, xVec, yVecHmin,
          panelWidth, panelHeight, xAxisLength, yAxisLength,
          minXData14, maxXData14, rangeXData14, minYData14, maxYData14, rangeYData14,
          RGBHexVHmin, '-', dSizeCnvs);


//Comparison to standard solar model from Table 3A of 
// ApJ, 387, 372 (1992) Guenther, D.B, Demarque, P., Kim, Y-C., Pinsonneault, M.
    if (ifStandardSun){
       var kSunRef = kRef();
       var logKSunRef = [];
       logKSunRef.length = numRef;
       for (var i=0; i<numRef; i++){
          logKSunRef[i] = log10E*Math.log(kSunRef[i]);
       }
       plot(cnvsFourteenId, rSunRef, logKSunRef,
          panelWidth, panelHeight, xAxisLength, yAxisLength,
          minXData14, maxXData14, rangeXData14, minYData14, maxYData14, rangeYData14,
          RGBHexSun, 'o', dSizeCnvs);
    }



    cnvsFourteenId.addEventListener("click", function() {
       //dataCoords(event, plotFourteenId);
       var xyString = dataCoords(event, cnvsFourteenId, xAxisLength, minXData14, rangeXData14, xAxisXCnvs,
                               yAxisLength, minYData14, rangeYData14, yAxisYCnvs);
       //console.log("PLOT TWO INSIDE: xAxisLength " + xAxisLength + " minXData " + minXData14 + " rangeXData " + rangeXData14 + " xAxisXCnvs " + xAxisXCnvs14);
       //txtPrint(xyString, titleOffsetX+1400, titleOffsetY+3140, 150, lineColor, plotFourteenId);
       txtSVGprint(xyString, titleOffsetX+200, titleOffsetY+320, lineColor, cnvsFourteenId);
    });


  } //End plot fourteen 



//
//  *****   PLOT TWELVE / PLOT 12
//
//

// Plot twelve: r vs nuclear energy mass genereation rate (epsPP, epsCNO) 
// 
    if ((ifShowStruc === true)) {

        var plotRow = 4;
        var plotCol = 2;
        var minXData = rvarMm[0];
        var maxXData = rvarMm[iUpper-1];
	//console.log("minXData " + minXData + " maxXData " + maxXData);
        //var xAxisName = "<span title='Rosseland mean optical depth'><a href='http://en.wikipedia.org/wiki/Optical_depth_%28astrophysics%29' target='_blank'>Log<sub>10</sub> <em>&#964</em><sub>Ros</sub></a></span>";
        var xAxisName = "<tspan style='font:italic;'>r</tspan><tspan style='baseline-shift:sub;'></tspan> (Mm)";
        var minYData = log10E*Math.log(epsPP[iCore-1]);
        var maxYData = log10E*Math.log(epsPP[1]);
	//console.log("minYData " + minYData + " maxYData " + maxYData);
        //var yAxisName = "<em>T</em><sub>Kin</sub> (K)";
        var yAxisName = "Log<tspan style='baseline-shift:sub;'>10</tspan> <tspan style='font:italic;'>\u03B5</tspan><tspan style='baseline-shift:sub;'>r</tspan> (erg/g/s)";
        var fineness = "normal";
					//JB
        //var panelOrigin = washer(plotRow, plotCol, wDefaultColor, plotTwelveId, cnvsTwelveId);
					//JB
        //panelX = panelOrigin[0];
        //panelY = panelOrigin[1];
        washerGrid(panelWidth, panelHeight,
            wDiskColor, plotTwelveId, cnvsTwelveId);
					//JB
	//cnvsTwelveId.setAttribute('fill', wDefaultColor);
        var xAxisParams = XAxis(panelWidth, panelHeight, xAxisLength, yAxisLength,
                minXData, maxXData, xAxisName, fineness,
                plotTwelveId, cnvsTwelveId);
        var cnvsCtx = xAxisParams[8];
        var yAxisParams = YAxis(panelWidth, panelHeight, xAxisLength, yAxisLength, 
                minYData, maxYData, yAxisName,fineness, plotTwelveId, cnvsTwelveId);
                                                
					//JB

        //
        //xOffset = xAxisParams[0];
        //yOffset = xAxisParams[4];
        var rangeXData12 = xAxisParams[1];
        var deltaXData12 = xAxisParams[2];
        var deltaXPxl12 = xAxisParams[3];
        var rangeYData12 = yAxisParams[1];
        var deltaYData12 = yAxisParams[2];
        var deltaYPxl12 = yAxisParams[3];
        //var xLowerYOffset = xAxisParams[5];
        var minXData12 = xAxisParams[6]; //updated value
        var minYData12 = yAxisParams[6]; //updated value
        var maxXData12 = xAxisParams[7]; //updated value
        var maxYData12 = yAxisParams[7]; //updated value    
        yFinesse = 0;       
        xFinesse = 0;       
        //
        //txtPrint("<span style='font-size:normal; color:blue'>Gas temperature </span>",
        //        titleOffsetX, titleOffsetY, 300, lineColor, plotTwelveId);
        txtSVGprint("log Mass power generation rate: <tspan style='fill:blue;><tspan style='font:italic;'>\u03B5<tspan style='baseline-shift:sub;'>PP</tspan></tspan> "
                +                " <tspan style='fill:red;><tspan style='font:italic;'>\u03B5<tspan style='baseline-shift:sub;'>CNO</tspan></tspan>",
                titleOffsetX, titleOffsetY, lineColor, cnvsTwelveId);
					//JB
        //var dSize = 5.0; //plot point size
        var dSizeCnvs = 1.0; //plot point size
        var opac = 1.0; //opacity
        // RGB color
        var r255 = 0;
        var g255 = 0;
        var b255 = 255; //blue

// Generally, plot every 10th depth:
	var lengthPlotV = Math.round(iUpper/10);
	//console.log("iUpper " + iUpper + " lengthPlotV " + lengthPlotV);
        var RGBHexVPP = [];
        RGBHexVPP.length = lengthPlotV;
        var RGBHexVCNO = [];
        RGBHexVCNO.length = lengthPlotV;
        var xVec = [];
        xVec.length = lengthPlotV;
        var yVecPP = [];
        yVecPP.length = lengthPlotV;
        var yVecCNO = [];
        yVecCNO.length = lengthPlotV;

    //Initialize plot argument arrays to valid value:
     for (var i = 0; i < lengthPlotV; i++){
        RGBHexVPP[i] = 0x000000;
        RGBHexVCNO[i] = 0x000000;
	xVec[i] = rvarMm[0];
	if (epsPP[2] > 0.0){
	   yVecPP[i] = log10E*Math.log(epsPP[iCore-1]);
	} else {
           yVecPP[i] = logTiny;
	}
	if (epsCNO[2] > 0.0){
	   yVecCNO[i] = log10E*Math.log(epsCNO[iCore-1]);
	} else {
           yVecCNO[i] = logTiny;
	}
     }

//        console.log("iUpper " + iUpper + " logRvar[iUpper-1] " + log10E*logRvar[iUpper-1] + " logRho[iUpper-1] " + log10E*logRho[iUpper-1]);
       j = 0;
       for (var i = 0; i < iUpper; i+=10){
           RGBHexVPP[j] = colHex(r255B, g255B, b255B);
           RGBHexVCNO[j] = colHex(r255R, g255R, b255R);
           //xVec[i] = log10E * logRvar[i];
           xVec[j] = rvarMm[i];
	   if (epsPP[i] > 0.0){
              yVecPP[j] = log10E * log10E*Math.log(epsPP[i]);
	   } else {
              yVecPP[j] = logTiny;
           }
	   if (epsCNO[i] > 0.0){
              yVecCNO[j] = log10E * log10E*Math.log(epsCNO[i]);
	   } else {
              yVecCNO[j] = logTiny;
           }
           //if ( (i%10) == 0 ){
           //  console.log("j " + j + " i " + i + " logRvar[i] " + log10E*logRvar[i] + " logTemp " + log10E*logTemp[i]);
	  // }
	   j++;
        }


     plot(cnvsTwelveId, xVec, yVecPP,
          panelWidth, panelHeight, xAxisLength, yAxisLength,
          minXData12, maxXData12, rangeXData12, minYData12, maxYData12, rangeYData12,
          RGBHexVPP, '-', dSizeCnvs);

     plot(cnvsTwelveId, xVec, yVecCNO,
          panelWidth, panelHeight, xAxisLength, yAxisLength,
          minXData12, maxXData12, rangeXData12, minYData12, maxYData12, rangeYData12,
          RGBHexVCNO, '-', dSizeCnvs);



    cnvsTwelveId.addEventListener("click", function() {
       //dataCoords(event, plotTwelveId);
       var xyString = dataCoords(event, cnvsTwelveId, xAxisLength, minXData12, rangeXData12, xAxisXCnvs,
                               yAxisLength, minYData12, rangeYData12, yAxisYCnvs);
       //console.log("PLOT TWO INSIDE: xAxisLength " + xAxisLength + " minXData " + minXData12 + " rangeXData " + rangeXData12 + " xAxisXCnvs " + xAxisXCnvs12);
       //txtPrint(xyString, titleOffsetX+400, titleOffsetY+340, 150, lineColor, plotTwelveId);
       txtSVGprint(xyString, titleOffsetX+200, titleOffsetY+320, lineColor, cnvsTwelveId);
    });


  } //End plot twelve 



// Detailed model output section:

//    
// Set up the canvas:
//

    // **********  Basic canvas parameters: These are numbers in px - needed for calculations:
    // All plots and other output must fit within this region to be white-washed between runs

    var xRangeT = 2250;
    var yRangeT = 20000;
    var xOffsetT = 10;
    var yOffsetT = 100;
    var charToPxT = 4; // width of typical character font in pixels - CAUTION: finesse!

    var zeroInt = 0;
    //these are the corresponding strings ready to be assigned to HTML style attributes


    var xRangeTStr = numToPxStrng(xRangeT);
    var yRangeTStr = numToPxStrng(yRangeT);
    var xOffsetTStr = numToPxStrng(xOffsetT);
    var yOffsetTStr = numToPxStrng(yOffsetT);
    // Very first thing on each load: White-wash the canvas!!

    // R & L_Bol:
    var colr = 0;
    var lineHeight = 17;
    var value, yTab;
    var vOffset = 60;
    var txtColor = "#000000"; //black


console.log("ifPrintStruc " + ifPrintStruc);
    if (ifPrintStruc == true) {

        var modelBanner = "Model: Mass " + massStar + " M_Sun, " + " Radius " + radius + " R_Sun, " + " Teff " + teff + " K, log(g) " + logg10 + " Y " + Yfrac + " Z " + Zfrac;
        txtPrint(modelBanner, 10, yOffsetT, 500, txtColor, printModelId);  
        txtPrint("Mean molecular weight (assumes complete ionization): " + muIamu + "(a.m.u.)", 10, yOffsetT + 2*lineHeight, 600, txtColor, printModelId);
        txtPrint("Polytropic constant, K: : " + K + ", polytropic index, n " + index + ", P<sub>gas</sub>/P " + beta, 10, yOffsetT + 3*lineHeight, 800, txtColor, printModelId);
        txtPrint("Radial structure", 10, yOffsetT + 4*lineHeight, 200, txtColor, printModelId);
        //Column headings:

        var xTab = 190;
        txtPrint("i", 10, yOffsetT + 6*lineHeight, 400, txtColor, printModelId);
        txtPrint("Radius (cm)", 10+xTab, yOffsetT + 6*lineHeight, 400, txtColor, printModelId);
        txtPrint("log<sub>10</sub> m<sub>r</sub> (cm)", 10 + 2*xTab, yOffsetT + 6*lineHeight, 400, txtColor, printModelId);
        txtPrint("log<sub>10</sub> <em>T</em><sub>Kin</sub> (K)", 10 + 3*xTab, yOffsetT + 6*lineHeight, 400, txtColor, printModelId);
        txtPrint("log<sub>10</sub> <em>P</em><sub>Gas</sub> (dynes cm<sup>-2</sup>)", 10 + 4*xTab, yOffsetT + 6*lineHeight, 400, txtColor, printModelId);
        txtPrint("log<sub>10</sub> <em>P</em><sub>Rad</sub> (dynes cm<sup>-2</sup>)", 10 + 5*xTab, yOffsetT + 6*lineHeight, 400, txtColor, printModelId);
        txtPrint("log<sub>10</sub> <em>&#961</em> (g cm<sup>-3</sup>)", 10 + 6*xTab, yOffsetT + 6*lineHeight, 400, txtColor, printModelId);
        txtPrint("log<sub>10</sub> L<sub>r</sub> (ergs)", 10 + 7*xTab, yOffsetT + 6*lineHeight, 400, txtColor, printModelId);

	j = 0;    
        for (var i = 0; i < iUpper; i+=10) {
            yTab = yOffsetT + vOffset + 7*lineHeight + (j+1)*lineHeight;
            numPrint(i, 10, yTab, txtColor, printModelId);
            value = rvar[i];
            value = value.toPrecision(5);
            numPrint(value, 10 + xTab, yTab, txtColor, printModelId);
            value = log10E * logMvar[i];
            value = value.toPrecision(5);
            numPrint(value, 10 + 2*xTab, yTab, txtColor, printModelId);
            value = log10E * logTemp[i];
            value = value.toPrecision(5);
            numPrint(value, 10 + 3*xTab, yTab, txtColor, printModelId);
            value = log10E * logPGas[i];
            value = value.toPrecision(5);
            numPrint(value, 10 + 4*xTab, yTab, txtColor, printModelId);
            value = log10E * logPRad[i];
            value = value.toPrecision(5);
            numPrint(value, 10 + 5*xTab, yTab, txtColor, printModelId);
            value = log10E * logRho[i];
            value = value.toPrecision(5);
            numPrint(value, 10 + 6*xTab, yTab, txtColor, printModelId);
            value = log10E * logLvar[i];
            value = value.toPrecision(5);
            numPrint(value, 10 + 7*xTab, yTab, txtColor, printModelId);
	    j++;
        }

    }

/*
    if (ifPrintSED == true) {

        var modelBanner = "Model: Teff " + teff + " K, log(g) " + logg10 + " log cm/s/s " + ", mass " + massStar + " M_Sun " + " X" + X + " Y " + Y + " Z " + Z;
        txtPrint(modelBanner, 10, yOffsetT, 500, txtColor, printModelId);  
        txtPrint("Monochromatic surface flux spectral energy distribution (SED)", 10, yOffsetT + lineHeight, 400, txtColor, printModelId);
        //Column headings:

        var xTab = 190;
        txtPrint("log<sub>10</sub> <em>&#955</em> (cm)", 10, yOffsetT + 2*lineHeight, 400, txtColor, printModelId);
        txtPrint("log<sub>10</sub> <em>F</em><sub>&#955</sub> (ergs s<sup>-1</sup> cm<sup>-2</sup> cm<sup>-1</sup>)", 10 + xTab, yOffsetT + 2*lineHeight, 400, txtColor, printModelId);
        for (var i = 0; i < numMaster; i++) {
            yTab = yOffsetT + vOffset + (i+1) * lineHeight;
            value = logE * Math.log(masterLams[i]);
            value = value.toPrecision(9);
            numPrint(value, 10, yTab, txtColor, printModelId);
            value = logE * masterFlux[1][i];
            value = value.toPrecision(7);
            numPrint(value, 10 + xTab, yTab, txtColor, printModelId);
        }
    }
*/


//
//
//  *******    END CODE
// 
//
    return;
}; //end function main()
